@@ -37,6 +37,7 @@
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
+#include <errno.h>
 #include "types.h"
 #include "common.h"
 #include "fat.h"
@@ -45,6 +46,7 @@
 #include "dir.h"
 #include "fat_dir.h"
 #include "log.h"
+#include "setdate.h"
 
 #define MSDOS_MKMODE(a,m) ((m & (a & ATTR_RO ? LINUX_S_IRUGO|LINUX_S_IXUGO : LINUX_S_IRWXUGO)) | (a & ATTR_DIR ? LINUX_S_IFDIR : LINUX_S_IFREG))
 struct fat_dir_struct
@@ -79,6 +81,10 @@ file_data_t *dir_fat_aux(const unsigned char*buffer, const unsigned int size, co
   file_data_t *current_file=NULL;
   unsigned int status;
   unsigned int inode;
+  int utf8=1;
+#ifdef HAVE_WCTOMB
+  wctomb(NULL, 0);
+#endif
 GetNew:
   status=0;
   long_slots = 0;
@@ -255,11 +261,29 @@ RecEnd:
       return dir_list;
     if((int8_t) unicode[0] != (int8_t) DELETED_FLAG)
     {
-      unsigned int i;
+      unsigned int i,o;
       file_data_t *new_file=(file_data_t *)MALLOC(sizeof(*new_file));
-      for(i=0;(unicode[i]!=0)&&(i<sizeof(new_file->name)-1);i++)
-	new_file->name[i]=(char) unicode[i];
-      new_file->name[i]=0;
+      for(i=0,o=0; unicode[i]!=0 && o<sizeof(new_file->name)-1; i++)
+      {
+	if(utf8 && unicode[i]>0x7f)
+	{
+#ifdef HAVE_WCTOMB
+	  const int sizec=wctomb(&new_file->name[o], unicode[i]);
+#else
+	  const int sizec=unicode[i];
+#endif
+	  if(sizec <= 0)
+	  {
+	    new_file->name[o]=(char) unicode[i];
+	    utf8=0;
+	  }
+	  else
+	    o += sizec;
+	}
+	else
+	  new_file->name[o++]=(char) unicode[i];
+      }
+      new_file->name[o]=0;
       new_file->stat.st_dev=0;
       new_file->stat.st_ino=inode;
       new_file->stat.st_mode = MSDOS_MKMODE(de->attr,(LINUX_S_IRWXUGO & ~(LINUX_S_IWGRP|LINUX_S_IWOTH)));
@@ -268,12 +292,15 @@ RecEnd:
       new_file->stat.st_gid=0;
       new_file->stat.st_rdev=0;
       new_file->stat.st_size=le32(de->size);
+#ifdef DJGPP
+      new_file->file_size=le32(de->size);
+#endif
 #ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
       new_file->stat.st_blksize=cluster_size;
 #ifdef HAVE_STRUCT_STAT_ST_BLOCKS
       if(new_file->stat.st_blksize!=0)
       {
-	new_file->stat.st_blocks=(new_file->stat.st_size+new_file->stat.st_blksize-1)/new_file->stat.st_blksize;
+	new_file->stat.st_blocks=(le32(de->size) + new_file->stat.st_blksize - 1) / new_file->stat.st_blksize;
       }
 #endif
 #endif
@@ -527,7 +554,11 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
   const unsigned int block_size=fat_sector_size(fat_header)*sectors_per_cluster;
   unsigned char *buffer_file=(unsigned char *)MALLOC(block_size);
   unsigned int cluster;
+#ifdef DJGPP
+  unsigned int file_size=file->file_size;
+#else
   unsigned int file_size=file->stat.st_size;
+#endif
   unsigned int fat_meth=FAT_FOLLOW_CLUSTER;
   uint64_t start_fat1,start_data,part_size;
   unsigned long int no_of_cluster,fat_length;
@@ -562,7 +593,7 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
     }
     if(fwrite(buffer_file, 1, toread, f_out) != toread)
     {
-      log_error("fat_copy: no space left on destination.\n");
+      log_error("fat_copy: failed to write data %s\n", strerror(errno));
       fclose(f_out);
       set_date(new_file, file->stat.st_atime, file->stat.st_mtime);
       free(new_file);
