@@ -70,12 +70,12 @@ static void file_check_doc(file_recovery_t *file_recovery)
   unsigned long int i;
   unsigned int freesect_count=0;  
   struct OLE_HDR *header=(struct OLE_HDR*)&buffer_header;
-  fseek(file_recovery->handle,0,SEEK_SET);
-  if(fread(&buffer_header,sizeof(buffer_header),1,file_recovery->handle)!=1)	/*reads first sector including OLE header */
-  {
-    file_recovery->file_size=0;
+  const uint64_t doc_file_size_org=file_recovery->file_size;
+  file_recovery->file_size=0;
+  /*reads first sector including OLE header */
+  if(fseek(file_recovery->handle, 0, SEEK_SET) < 0 ||
+      fread(&buffer_header, sizeof(buffer_header), 1, file_recovery->handle) != 1)
     return ;
-  }
 #ifdef DEBUG_OLE
   log_info("file_check_doc %s\n", file_recovery->filename);
   log_trace("num_FAT_blocks       %u\n",le32(header->num_FAT_blocks));
@@ -85,29 +85,22 @@ static void file_check_doc(file_recovery_t *file_recovery)
   if(le32(header->num_FAT_blocks)==0 ||
       le32(header->num_extra_FAT_blocks)>50 ||
       le32(header->num_FAT_blocks)>109+le32(header->num_extra_FAT_blocks)*((1<<le16(header->uSectorShift))-1))
-  {
-    file_recovery->file_size=0;
     return ;
-  }
   if((fat=OLE_load_FAT(file_recovery->handle, header))==NULL)
-  {
-    file_recovery->file_size=0;
     return ;
-  }
   /* Search how many entries are not used at the end of the FAT */
   for(i=(le32(header->num_FAT_blocks)<<le16(header->uSectorShift))/4-1;
       i>((le32(header->num_FAT_blocks)-1)<<le16(header->uSectorShift))/4 && le32(fat[i])==0xFFFFFFFF;
       i--)
     freesect_count++;
   doc_file_size=512+(((le32(header->num_FAT_blocks)<<le16(header->uSectorShift))/4-freesect_count)<<le16(header->uSectorShift));
-  if(doc_file_size > file_recovery->file_size)
+  if(doc_file_size > doc_file_size_org)
   {
     free(fat);
     return ;
   }
-  file_recovery->file_size=doc_file_size;
 #ifdef DEBUG_OLE
-  log_trace("==> size : %llu\n",(long long unsigned)file_recovery->file_size);
+  log_trace("==> size : %llu\n", (long long unsigned)doc_file_size);
 #endif
   {
     unsigned int block;
@@ -124,27 +117,23 @@ static void file_check_doc(file_recovery_t *file_recovery)
 	block!=0xFFFFFFFE && i<fat_entries;
 	block=le32(fat[block]), i++)
     {
-      struct OLE_DIR *dir_entries=(struct OLE_DIR *)MALLOC(1<<le16(header->uSectorShift));
+      struct OLE_DIR *dir_entries;
 #ifdef DEBUG_OLE
       log_info("read block %u\n", block);
 #endif
       if(!(block < fat_entries))
       {
-	file_recovery->file_size=0;
-	free(dir_entries);
 	free(fat);
 	return ;
       }
       if(fseek(file_recovery->handle, 512+(block<<le16(header->uSectorShift)), SEEK_SET)<0)
       {
-	file_recovery->file_size=0;
-	free(dir_entries);
 	free(fat);
 	return ;
       }
+      dir_entries=(struct OLE_DIR *)MALLOC(1<<le16(header->uSectorShift));
       if(fread(dir_entries, (1<<le16(header->uSectorShift)), 1, file_recovery->handle)!=1)
       {
-	file_recovery->file_size=0;
 	free(dir_entries);
 	free(fat);
 	return ;
@@ -157,9 +146,8 @@ static void file_check_doc(file_recovery_t *file_recovery)
 	    sid++,dir_entry++)
 	{
 	  if(le32(dir_entry->start_block) > 0 && le32(dir_entry->size) > 0 &&
-	      (le32(dir_entry->start_block) > fat_entries || le32(dir_entry->size) > file_recovery->file_size))
+	      (le32(dir_entry->start_block) > fat_entries || le32(dir_entry->size) > doc_file_size))
 	  {
-	    file_recovery->file_size=0;
 	    free(dir_entries);
 	    free(fat);
 	    return ;
@@ -170,6 +158,7 @@ static void file_check_doc(file_recovery_t *file_recovery)
     }
   }
   free(fat);
+  file_recovery->file_size=doc_file_size;
 }
 
 static const char *ole_get_file_extension(const unsigned char *buffer, const unsigned int buffer_size)
@@ -189,9 +178,9 @@ static const char *ole_get_file_extension(const unsigned char *buffer, const uns
   else
   {
     const uint32_t *fati=(const uint32_t *)(header+1);
-    const unsigned int fat_offset=(le32(fati[0])<<header->uSectorShift)+512;
+    const unsigned int fat_offset=(le32(fati[0]) << le16(header->uSectorShift))+512;
     fat=(const uint32_t *)&buffer[fat_offset];
-    fat_entries=(le32(header->num_FAT_blocks) << header->uSectorShift)/4;
+    fat_entries=(le32(header->num_FAT_blocks) << le16(header->uSectorShift))/4;
     if(fat_offset>buffer_size)
       fat_entries=0;
     else if(fat_offset+fat_entries>buffer_size)
@@ -223,7 +212,7 @@ static const char *ole_get_file_extension(const unsigned char *buffer, const uns
       {
 #ifdef DEBUG_OLE
 	unsigned int j;
-	for(j=0;j<64 && dir_entry->name[j]!='\0' && j<dir_entry->namsiz;j+=2)
+	for(j=0;j<64 && j<dir_entry->namsiz && dir_entry->name[j]!='\0';j+=2)
 	{
 	  log_info("%c",dir_entry->name[j]);
 	}
@@ -311,12 +300,13 @@ static int header_check_doc(const unsigned char *buffer, const unsigned int buff
     const struct OLE_HDR *header=(const struct OLE_HDR *)buffer;
     if(le16(header->reserved)!=0 || le32(header->reserved1)!=0)
       return 0;
-    /* qbb file have reserved2=4 */
-    if(le32(header->reserved2)!=0 && le32(header->reserved2)!=4)
+    /* max file have reserved2=1
+     * qbb file have reserved2=4 */
+    if(le32(header->reserved2)!=0 && le32(header->reserved2)!=1 && le32(header->reserved2)!=4)
       return 0;
     if(le16(header->uMiniSectorShift)!=6)
       return 0;
-    /* qbb file have uSectorShift=12 */
+    /* max and qbb file have uSectorShift=12 */
     if(le16(header->uSectorShift)!=9 && le16(header->uSectorShift)!=12)
       return 0;
     /*
@@ -542,16 +532,23 @@ static uint64_t get64u(const void *buffer, const unsigned int offset)
 
 static const char *software2ext(const unsigned int count, const unsigned char *software)
 {
+  if(count>=12 && memcmp(software, "MicroStation", 12)==0)
+    return "dgn";
   if(count>=14 && memcmp(software, "Microsoft Word", 14)==0)
     return "doc";
-  if(count>=21 && memcmp(software, "Microsoft Office Word", 21)==0)
-    return "doc";
   if(count>=15 && memcmp(software, "Microsoft Excel", 15)==0)
     return "xls";
   if(count>=20 && memcmp(software, "Microsoft PowerPoint", 20)==0)
     return "ppt";
-  if(count>=12 && memcmp(software, "MicroStation", 12)==0)
-    return "dgn";
+  if(count>=21 && memcmp(software, "Microsoft Office Word", 21)==0)
+    return "doc";
+  return NULL;
+}
+
+static const char *software_uni2ext(const unsigned int count, const unsigned char *software)
+{
+  if(count>=17 && memcmp(software, "D\0e\0l\0c\0a\0m\0 \0P\0o\0w\0e\0r\0S\0H\0A\0P\0E\0", 34)==0)
+    return "psmodel";
   return NULL;
 }
 
@@ -610,6 +607,25 @@ static void OLE_parse_summary_aux(const unsigned char *dataPt, const unsigned in
 #endif
 	*ext=software2ext(count, &dataPt[valStart + 4]);
       }
+      /* tag: Software, type: VT_LPWSTR */
+      if(tag==0x12 && type==31)
+      {
+	unsigned int count=get32u(dataPt, valStart);
+	if(valStart + 4 + 2 * count > dirLen)
+	  return ;
+#ifdef DEBUG_OLE
+	{
+	  unsigned int j;
+	  log_info("Software ");
+	  for(j=0; j < 2 * count; j+=2)
+	  {
+	    log_info("%c", dataPt[valStart + 4 + j]);
+	  }
+	  log_info("\n");
+	}
+#endif
+	*ext=software_uni2ext(count, &dataPt[valStart + 4]);
+      }
       if(tag==0x02 && type==30 && *title==NULL)
       {
 	const unsigned int count=get32u(dataPt, valStart);
@@ -665,11 +681,11 @@ static void OLE_parse_summary(FILE *file, const uint32_t *fat, const unsigned in
     const unsigned int block, const unsigned int len, const char **ext, char **title, time_t *file_time)
 {
   unsigned char *summary=NULL;
-  if(len < 48 || len>102400)
+  if(len < 48 || len>1024*1024)
     return ;
   if(len < le32(header->miniSectorCutoff))
   {
-    if(le32(header->csectMiniFat)!=0 && ministream_size > 0 && ministream_size < 102400)
+    if(le32(header->csectMiniFat)!=0 && ministream_size > 0 && ministream_size < 1024*1024)
     {
       const unsigned int mini_fat_entries=(le32(header->csectMiniFat) << le16(header->uSectorShift)) / 4;
       uint32_t *minifat;
@@ -715,8 +731,9 @@ static void file_rename_doc(const char *old_filename)
 #ifdef DEBUG_OLE
   log_info("file_rename_doc(%s)\n", old_filename);
 #endif
-  fseek(file,0,SEEK_SET);
-  if(fread(&buffer_header,sizeof(buffer_header),1,file)!=1)	/*reads first sector including OLE header */
+  /*reads first sector including OLE header */
+  if(fseek(file, 0, SEEK_SET) < 0 ||
+      fread(&buffer_header, sizeof(buffer_header), 1, file) != 1)
   {
     fclose(file);
     return ;
@@ -732,7 +749,10 @@ static void file_rename_doc(const char *old_filename)
   if(le16(header->uSectorShift)==12)
   {
     fclose(file);
-    file_rename(old_filename, NULL, 0, 0, "qbb", 1);
+    if(le32(header->reserved2)==1)
+      file_rename(old_filename, NULL, 0, 0, "max", 1);
+    else
+      file_rename(old_filename, NULL, 0, 0, "qbb", 1);
     return ;
   }
   if((fat=OLE_load_FAT(file, header))==NULL)
@@ -757,14 +777,14 @@ static void file_rename_doc(const char *old_filename)
 	block<fat_entries && block!=0xFFFFFFFE && i<fat_entries;
 	block=le32(fat[block]), i++)
     {
-      struct OLE_DIR *dir_entries=(struct OLE_DIR *)MALLOC(1<<le16(header->uSectorShift));
+      struct OLE_DIR *dir_entries;
       if(fseek(file, 512+(block<<le16(header->uSectorShift)), SEEK_SET)<0)
       {
 	free(fat);
-	free(dir_entries);
 	fclose(file);
 	return ;
       }
+      dir_entries=(struct OLE_DIR *)MALLOC(1<<le16(header->uSectorShift));
       if(fread(dir_entries, (1<<le16(header->uSectorShift)), 1, file)!=1)
       {
 	free(fat);
@@ -810,7 +830,7 @@ static void file_rename_doc(const char *old_filename)
 	    };
 #ifdef DEBUG_OLE
 	    unsigned int j;
-	    for(j=0;j<64 && dir_entry->name[j]!='\0' && j<dir_entry->namsiz;j+=2)
+	    for(j=0;j<64 && j<dir_entry->namsiz && dir_entry->name[j]!='\0';j+=2)
 	    {
 	      log_info("%c",dir_entry->name[j]);
 	    }
@@ -883,8 +903,10 @@ static void file_rename_doc(const char *old_filename)
 	    /* Page Maker */
 	    else if(memcmp(&dir_entry->name, "P\0a\0g\0e\0M\0a\0k\0e\0r\0", 18)==0)
 	      ext="p65";
-	    if(sid==1 && memcmp(dir_entry->name, "D\0g\0n", 6)==0)
+	    else if(sid==1 && memcmp(dir_entry->name, "D\0g\0n", 6)==0)
 	      ext="dgn";
+	    else if(sid==1 && memcmp(dir_entry->name, "d\0o\0c\0.\0d\0e\0t\0", 14)==0)
+	      ext="psmodel";
 	  }
 	}
       }
