@@ -40,7 +40,7 @@ const file_hint_t file_hint_flac= {
   .min_header_distance=0,
   .max_filesize=100*1024*1024,
   .recover=1,
-  .header_check=&header_check_flac,
+  .enable_by_default=1,
 	.register_header_check=&register_header_check_flac
 };
 
@@ -56,7 +56,11 @@ static int header_check_flac(const unsigned char *buffer, const unsigned int buf
   if(memcmp(buffer, flac_header, sizeof(flac_header))==0)
   {
     reset_file_recovery(file_recovery_new);
+#ifdef DJGPP
+    file_recovery_new->extension="flc";
+#else
     file_recovery_new->extension=file_hint_flac.extension;
+#endif
     return 1;
   }
   return 0;
