@@ -39,12 +39,6 @@
 #include <setjmp.h>
 #endif
 #ifdef HAVE_JPEGLIB_H
-#ifdef __MINGW32__
-#ifndef HAVE_BOOLEAN
-#define HAVE_BOOLEAN
-typedef int boolean;
-#endif
-#endif
 #include <jpeglib.h>
 #include "suspend.h"
 #endif
@@ -91,8 +85,36 @@ static void register_header_check_jpg(file_stat_t *file_stat)
   register_header_check(0, jpg_header_com,sizeof(jpg_header_com), &header_check_jpg, file_stat);
 }
 
+static void jpg_get_size(const unsigned char *buffer, const unsigned int buffer_size, unsigned int *height, unsigned int *width)
+{
+  unsigned int i=2;
+  while(i+8<buffer_size)
+  {
+    if(buffer[i]==0xFF && buffer[i+1]==0xFF)
+      i++;
+    else if(buffer[i]==0xFF)
+    {
+      const unsigned int size=(buffer[i+2]<<8)+buffer[i+3];
+      if(buffer[i+1]==0xc0)	/* SOF0 */
+      {
+	*height=(buffer[i+5]<<8)+buffer[i+6];
+	*width=(buffer[i+7]<<8)+buffer[i+8];
+	return;
+      }
+      i+=2+size;
+    }
+    else
+    {
+      return;
+    }
+  }
+}
+
 static int header_check_jpg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
+  unsigned int width=0;
+  unsigned int height=0;
+  jpg_get_size(buffer, buffer_size, &height, &width);
   if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
       file_recovery->file_stat->file_hint==&file_hint_indd)
     return 0;
@@ -107,6 +129,24 @@ static int header_check_jpg(const unsigned char *buffer, const unsigned int buff
 	(long long unsigned)file_recovery->file_size);
     return 0;
   }
+  /* Don't recover the thumb instead of the jpg itself */
+  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+      file_recovery->file_stat->file_hint==&file_hint_jpg &&
+      file_recovery->file_size <= 4096 &&
+      memcmp(buffer, jpg_header_app0, sizeof(jpg_header_app0))==0 &&
+      width>0 && width<200 && height>0 && height<200)
+  {
+    return 0;
+  }
+  /* Some JPG have two APP1 markers, avoid to dicard the first one */
+  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+      file_recovery->file_stat->file_hint==&file_hint_jpg &&
+      memcmp(buffer, jpg_header_app1, sizeof(jpg_header_app1))==0 &&
+      memcmp(&buffer[6], "http://ns.adobe.com/xap/", 24)==0)
+  {
+    return 0;
+  }
+
   /* Don't extract jpg inside AVI */
   if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
       file_recovery->file_stat->file_hint==&file_hint_riff &&
@@ -190,7 +230,7 @@ typedef struct {
 
   FILE * infile;		/* source stream */
   JOCTET * buffer;		/* start of buffer */
-  boolean start_of_file;	/* have we gotten any data yet? */
+  int start_of_file;	/* have we gotten any data yet? */
   unsigned long int offset;
   unsigned long int file_size;
   unsigned long int file_size_max;
@@ -298,7 +338,7 @@ static void jpg_init_source (j_decompress_ptr cinfo)
  * the front of the buffer rather than discarding it.
  */
 
-static boolean jpg_fill_input_buffer (j_decompress_ptr cinfo)
+static int jpg_fill_input_buffer (j_decompress_ptr cinfo)
 {
   my_source_mgr * src = (my_source_mgr *) cinfo->src;
   size_t nbytes;
@@ -481,7 +521,8 @@ static inline int jpeg_session_resume(struct jpeg_session_struct *jpeg_session)
   if(resume_memory((j_common_ptr)&jpeg_session->cinfo))
     return -1;
   src = (my_source_mgr *) jpeg_session->cinfo.src;
-  fseek(jpeg_session->handle, jpeg_session->offset + src->file_size, SEEK_SET);
+  if(fseek(jpeg_session->handle, jpeg_session->offset + src->file_size, SEEK_SET) < 0)
+    return -1;
   return 0;
 }
 
@@ -1015,7 +1056,8 @@ static void jpg_search_marker(file_recovery_t *file_recovery)
     return ;
   offset=file_recovery->offset_error / file_recovery->blocksize * file_recovery->blocksize;
   i=file_recovery->offset_error % file_recovery->blocksize;
-  fseek(infile, offset, SEEK_SET);
+  if(fseek(infile, offset, SEEK_SET) < 0)
+    return ;
   do
   {
     while((nbytes=fread(&buffer, 1, sizeof(buffer), infile))>0)
@@ -1056,7 +1098,8 @@ static uint64_t jpg_check_structure(file_recovery_t *file_recovery, const unsign
   uint64_t thumb_offset=0;
   int nbytes;
   file_recovery->extra=0;
-  fseek(infile, 0, SEEK_SET);
+  if(fseek(infile, 0, SEEK_SET) < 0)
+    return 0;
   if((nbytes=fread(&buffer, 1, sizeof(buffer), infile))>0)
   {
     unsigned int offset;
@@ -1064,7 +1107,7 @@ static uint64_t jpg_check_structure(file_recovery_t *file_recovery, const unsign
     for(offset=file_recovery->blocksize; offset < nbytes && file_recovery->offset_error==0; offset+=file_recovery->blocksize)
     {
       if(buffer[offset]==0xff && buffer[offset+1]==0xd8 && buffer[offset+2]==0xff &&
-	(buffer[offset+3]==0xe0 || buffer[offset+3]==0xe1 || buffer[offset+3]==0xec))
+	(buffer[offset+3]==0xe1 || buffer[offset+3]==0xec))
       {
 	file_recovery->offset_error=offset;
       }
