@@ -56,7 +56,9 @@ extern const file_hint_t file_hint_blend;
 extern const file_hint_t file_hint_bmp;
 extern const file_hint_t file_hint_bz2;
 extern const file_hint_t file_hint_cab;
+extern const file_hint_t file_hint_caf;
 extern const file_hint_t file_hint_cam;
+extern const file_hint_t file_hint_catdrawing;
 extern const file_hint_t file_hint_cdt;
 extern const file_hint_t file_hint_chm;
 extern const file_hint_t file_hint_class;
@@ -67,9 +69,11 @@ extern const file_hint_t file_hint_crw;
 extern const file_hint_t file_hint_ctg;
 extern const file_hint_t file_hint_cwk;
 extern const file_hint_t file_hint_d2s;
+extern const file_hint_t file_hint_dar;
 extern const file_hint_t file_hint_dat;
 extern const file_hint_t file_hint_dbf;
 extern const file_hint_t file_hint_dbn;
+extern const file_hint_t file_hint_ddf;
 extern const file_hint_t file_hint_dex;
 extern const file_hint_t file_hint_dim;
 extern const file_hint_t file_hint_dir;
@@ -115,6 +119,7 @@ extern const file_hint_t file_hint_fp7;
 extern const file_hint_t file_hint_freeway;
 extern const file_hint_t file_hint_frm;
 extern const file_hint_t file_hint_fs;
+extern const file_hint_t file_hint_gam;
 extern const file_hint_t file_hint_gct;
 extern const file_hint_t file_hint_gho;
 extern const file_hint_t file_hint_gif;
@@ -122,6 +127,7 @@ extern const file_hint_t file_hint_gm6;
 extern const file_hint_t file_hint_gp5;
 extern const file_hint_t file_hint_gpg;
 extern const file_hint_t file_hint_gz;
+extern const file_hint_t file_hint_hdf;
 extern const file_hint_t file_hint_hds;
 extern const file_hint_t file_hint_hr9;
 extern const file_hint_t file_hint_icc;
@@ -138,6 +144,7 @@ extern const file_hint_t file_hint_ldf;
 extern const file_hint_t file_hint_logic;
 extern const file_hint_t file_hint_lnk;
 extern const file_hint_t file_hint_lso;
+extern const file_hint_t file_hint_lxo;
 extern const file_hint_t file_hint_lzo;
 extern const file_hint_t file_hint_m2ts;
 extern const file_hint_t file_hint_mat;
@@ -163,15 +170,18 @@ extern const file_hint_t file_hint_nds;
 extern const file_hint_t file_hint_njx;
 extern const file_hint_t file_hint_nk2;
 extern const file_hint_t file_hint_nsf;
+extern const file_hint_t file_hint_oci;
 extern const file_hint_t file_hint_ogg;
 extern const file_hint_t file_hint_one;
 extern const file_hint_t file_hint_orf;
 extern const file_hint_t file_hint_paf;
 extern const file_hint_t file_hint_pap;
+extern const file_hint_t file_hint_par2;
 extern const file_hint_t file_hint_pcap;
 extern const file_hint_t file_hint_pct;
 extern const file_hint_t file_hint_pcx;
 extern const file_hint_t file_hint_pdf;
+extern const file_hint_t file_hint_pds;
 extern const file_hint_t file_hint_pfx;
 extern const file_hint_t file_hint_png;
 extern const file_hint_t file_hint_pnm;
@@ -205,6 +215,7 @@ extern const file_hint_t file_hint_rm;
 extern const file_hint_t file_hint_rns;
 extern const file_hint_t file_hint_rpm;
 extern const file_hint_t file_hint_rw2;
+extern const file_hint_t file_hint_save;
 extern const file_hint_t file_hint_ses;
 extern const file_hint_t file_hint_sib;
 extern const file_hint_t file_hint_sig;
@@ -224,6 +235,7 @@ extern const file_hint_t file_hint_tar;
 extern const file_hint_t file_hint_tax;
 extern const file_hint_t file_hint_tib;
 extern const file_hint_t file_hint_tiff;
+extern const file_hint_t file_hint_tivo;
 extern const file_hint_t file_hint_torrent;
 extern const file_hint_t file_hint_tph;
 extern const file_hint_t file_hint_ts;
@@ -235,11 +247,13 @@ extern const file_hint_t file_hint_vdi;
 extern const file_hint_t file_hint_veg;
 extern const file_hint_t file_hint_vfb;
 extern const file_hint_t file_hint_vmdk;
+extern const file_hint_t file_hint_vmg;
 extern const file_hint_t file_hint_wdp;
 extern const file_hint_t file_hint_win;
 extern const file_hint_t file_hint_wks;
 extern const file_hint_t file_hint_wmf;
 extern const file_hint_t file_hint_wnk;
+extern const file_hint_t file_hint_wpb;
 extern const file_hint_t file_hint_wpd;
 extern const file_hint_t file_hint_wtv;
 extern const file_hint_t file_hint_wv;
@@ -284,7 +298,9 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_bmp  },
   { .enable=0, .file_hint=&file_hint_bz2  },
   { .enable=0, .file_hint=&file_hint_cab  },
+  { .enable=0, .file_hint=&file_hint_caf  },
   { .enable=0, .file_hint=&file_hint_cam  },
+  { .enable=0, .file_hint=&file_hint_catdrawing  },
   { .enable=0, .file_hint=&file_hint_cdt  },
   { .enable=0, .file_hint=&file_hint_chm  },
   { .enable=0, .file_hint=&file_hint_class },
@@ -295,9 +311,11 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_ctg  },
   { .enable=0, .file_hint=&file_hint_cwk  },
   { .enable=0, .file_hint=&file_hint_d2s  },
+  { .enable=0, .file_hint=&file_hint_dar  },
   { .enable=0, .file_hint=&file_hint_dat  },
   { .enable=0, .file_hint=&file_hint_dbf  },
   { .enable=0, .file_hint=&file_hint_dbn  },
+  { .enable=0, .file_hint=&file_hint_ddf  },
   { .enable=0, .file_hint=&file_hint_dex  },
   { .enable=0, .file_hint=&file_hint_dim  },
   { .enable=0, .file_hint=&file_hint_dir  },
@@ -342,6 +360,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_freeway  },
   { .enable=0, .file_hint=&file_hint_frm  },
   { .enable=0, .file_hint=&file_hint_fs   },
+  { .enable=0, .file_hint=&file_hint_gam  },
   { .enable=0, .file_hint=&file_hint_gct  },
   { .enable=0, .file_hint=&file_hint_gho  },
   { .enable=0, .file_hint=&file_hint_gif  },
@@ -349,6 +368,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_gp5  },
   { .enable=0, .file_hint=&file_hint_gpg  },
   { .enable=0, .file_hint=&file_hint_gz   },
+  { .enable=0, .file_hint=&file_hint_hdf  },
   { .enable=0, .file_hint=&file_hint_hds  },
   { .enable=0, .file_hint=&file_hint_hr9  },
   { .enable=0, .file_hint=&file_hint_icc  },
@@ -365,6 +385,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_logic},
   { .enable=0, .file_hint=&file_hint_lnk  },
   { .enable=0, .file_hint=&file_hint_lso  },
+  { .enable=0, .file_hint=&file_hint_lxo  },
   { .enable=0, .file_hint=&file_hint_lzo  },
   { .enable=0, .file_hint=&file_hint_m2ts },
   { .enable=0, .file_hint=&file_hint_mat  },
@@ -390,15 +411,18 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_njx  },
   { .enable=0, .file_hint=&file_hint_nk2  },
   { .enable=0, .file_hint=&file_hint_nsf  },
+  { .enable=0, .file_hint=&file_hint_oci  },
   { .enable=0, .file_hint=&file_hint_ogg  },
   { .enable=0, .file_hint=&file_hint_one  },
   { .enable=0, .file_hint=&file_hint_orf  },
   { .enable=0, .file_hint=&file_hint_paf  },
   { .enable=0, .file_hint=&file_hint_pap  },
+  { .enable=0, .file_hint=&file_hint_par2  },
   { .enable=0, .file_hint=&file_hint_pcap },
   { .enable=0, .file_hint=&file_hint_pct  },
   { .enable=0, .file_hint=&file_hint_pcx  },
   { .enable=0, .file_hint=&file_hint_pdf  },
+  { .enable=0, .file_hint=&file_hint_pds  },
   { .enable=0, .file_hint=&file_hint_pfx  },
   { .enable=0, .file_hint=&file_hint_png  },
   { .enable=0, .file_hint=&file_hint_pnm  },
@@ -432,6 +456,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_rns  },
   { .enable=0, .file_hint=&file_hint_rpm  },
   { .enable=0, .file_hint=&file_hint_rw2  },
+  { .enable=0, .file_hint=&file_hint_save  },
   { .enable=0, .file_hint=&file_hint_ses  },
   { .enable=0, .file_hint=&file_hint_sib  },
   { .enable=0, .file_hint=&file_hint_sit  },
@@ -450,6 +475,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_tax  },
   { .enable=0, .file_hint=&file_hint_tib  },
   { .enable=0, .file_hint=&file_hint_tiff },
+  { .enable=0, .file_hint=&file_hint_tivo  },
   { .enable=0, .file_hint=&file_hint_torrent  },
   { .enable=0, .file_hint=&file_hint_tph  },
   { .enable=0, .file_hint=&file_hint_ts   },
@@ -459,14 +485,16 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_tz   },
   { .enable=0, .file_hint=&file_hint_vault  },
   { .enable=0, .file_hint=&file_hint_vfb  },
-  { .enable=0, .file_hint=&file_hint_vmdk },
   { .enable=0, .file_hint=&file_hint_vdi  },
   { .enable=0, .file_hint=&file_hint_veg  },
+  { .enable=0, .file_hint=&file_hint_vmdk },
+  { .enable=0, .file_hint=&file_hint_vmg  },
   { .enable=0, .file_hint=&file_hint_wdp  },
   { .enable=0, .file_hint=&file_hint_win  },
   { .enable=0, .file_hint=&file_hint_wks  },
   { .enable=0, .file_hint=&file_hint_wmf  },
   { .enable=0, .file_hint=&file_hint_wnk  },
+  { .enable=0, .file_hint=&file_hint_wpb  },
   { .enable=0, .file_hint=&file_hint_wpd  },
   { .enable=0, .file_hint=&file_hint_wtv  },
   { .enable=0, .file_hint=&file_hint_wv   },
