@@ -143,11 +143,20 @@ static const unsigned char header_reg[8]  	= "REGEDIT4";
 static const unsigned char header_ReturnPath[13]= {'R','e','t','u','r','n','-','P','a','t','h',':',' '};
 static const unsigned char header_rpp[16]	= { '<', 'R', 'E', 'A', 'P', 'E', 'R', '_', 'P', 'R', 'O', 'J', 'E', 'C', 'T', ' '};
 static const unsigned char header_rtf[5]	= { '{','\\','r','t','f'};
+static const unsigned char header_seenezsst[8]=  {
+  0x23, 'S' , 'e' , 'e' , 'N' , 'e' , 'z' , ' ' ,
+};
 /* firefox session store */
 static const unsigned char header_sessionstore[42]  	= "({\"windows\":[{\"tabs\":[{\"entries\":[{\"url\":\"";
 static const unsigned char header_sh[9]  	= "#!/bin/sh";
 static const unsigned char header_slk[10]  	= "ID;PSCALC3";
 static const unsigned char header_smil[6]  	= "<smil>";
+static const unsigned char header_snz_unix[8]=  {
+  'D' , 'E' , 'F' , 'A' , 'U' , 'L' , 'T' ,'\n'
+};
+static const unsigned char header_snz_win[9]=  {
+  'D' , 'E' , 'F' , 'A' , 'U' , 'L' , 'T' ,'\r', '\n'
+};
 static const unsigned char header_stl[6]	= "solid ";
 static const unsigned char header_stp[13]  	= "ISO-10303-21;";
 static const unsigned char header_ttd[55]	= "FF 09 FF FF FF FF FF FF FF FF FF FF FF FF FF FF FFFF 00";
@@ -170,6 +179,12 @@ static const char sign_java1[6]			= "class";
 static const char sign_java3[15]		= "private static";
 static const char sign_java4[17]		= "public interface";
 static unsigned char ascii_char[256];
+static const unsigned char hdr_header[17]=  {
+  'E' , 'N' , 'V' , 'I' , 0x0d, 0x0a, 'd' , 'e' ,
+  's' , 'c' , 'r' , 'i' , 'p' , 't' , 'i' , 'o' ,
+  'n' 
+};
+
 
 static void register_header_check_txt(file_stat_t *file_stat)
 {
@@ -197,6 +212,7 @@ static void register_header_check_fasttxt(file_stat_t *file_stat)
   register_header_check(0, header_dif, sizeof(header_dif), &header_check_fasttxt, file_stat);
   register_header_check(0, header_emka, sizeof(header_emka), &header_check_fasttxt, file_stat);
   register_header_check(0, header_ers,sizeof(header_ers), &header_check_fasttxt, file_stat);
+  register_header_check(0, hdr_header, sizeof(hdr_header), &header_check_fasttxt, file_stat);
   register_header_check(0, header_html, sizeof(header_html), &header_check_fasttxt, file_stat);
   register_header_check(0, header_ics, sizeof(header_ics), &header_check_fasttxt, file_stat);
   register_header_check(0, header_imm,sizeof(header_imm), &header_check_fasttxt, file_stat);
@@ -222,10 +238,13 @@ static void register_header_check_fasttxt(file_stat_t *file_stat)
   register_header_check(0, header_ReturnPath,sizeof(header_ReturnPath), &header_check_fasttxt, file_stat);
   register_header_check(0, header_rpp,sizeof(header_rpp), &header_check_fasttxt, file_stat);
   register_header_check(0, header_rtf,sizeof(header_rtf), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_seenezsst, sizeof(header_seenezsst), &header_check_fasttxt, file_stat);
   register_header_check(0, header_sessionstore, sizeof(header_sessionstore), &header_check_fasttxt, file_stat);
   register_header_check(0, header_sh,sizeof(header_sh), &header_check_fasttxt, file_stat);
   register_header_check(0, header_slk,sizeof(header_slk), &header_check_fasttxt, file_stat);
   register_header_check(0, header_smil,sizeof(header_smil), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_snz_unix, sizeof(header_snz_unix), &header_check_fasttxt, file_stat);
+  register_header_check(0, header_snz_win, sizeof(header_snz_win), &header_check_fasttxt, file_stat);
   register_header_check(0, header_stl,sizeof(header_stl), &header_check_fasttxt, file_stat);
   register_header_check(0, header_stp,sizeof(header_stp), &header_check_fasttxt, file_stat);
   register_header_check(0, header_ttd, sizeof(header_ttd), &header_check_fasttxt, file_stat);
@@ -590,6 +609,23 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     file_recovery_new->extension="slk";
     return 1;
   }
+  if(memcmp(buffer,header_seenezsst,sizeof(header_seenezsst))==0)
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="SeeNezSST";
+    return 1;
+  }
+  if(memcmp(buffer,header_snz_unix,sizeof(header_snz_unix))==0 ||
+      memcmp(buffer,header_snz_win,sizeof(header_snz_win))==0)
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="snz";
+    return 1;
+  }
   if(memcmp(buffer, header_mysql, sizeof(header_mysql))==0 ||
       memcmp(buffer, header_phpMyAdmin, sizeof(header_phpMyAdmin))==0 ||
       memcmp(buffer, header_postgreSQL, sizeof(header_postgreSQL))==0 ||
@@ -621,6 +657,15 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     file_recovery_new->extension="ers";
     return 1;
   }
+  if(memcmp(buffer, hdr_header, sizeof(hdr_header))==0)
+  {
+    /* ENVI */
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="hdr";
+    return 1;
+  }
   if(memcmp(buffer, header_emka, sizeof(header_emka))==0)
   {
     /* EMKA IOX file */
@@ -702,6 +747,11 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
     }
     else if(td_memmem(buffer, buffer_size, "QBFSD", 5)!=NULL)
       file_recovery_new->extension="fst";
+    else if(td_memmem(buffer, buffer_size, "<collection type=\"GC", 20)!=NULL)
+    {
+      /* GCstart, personal collections manager, http://www.gcstar.org/ */
+      file_recovery_new->extension="gcs";
+    }
     else if(td_memmem(buffer, buffer_size, "<html", 5)!=NULL)
     {
 #ifdef DJGPP
@@ -1111,11 +1161,11 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       unsigned int i;
       memset(&stats, 0, sizeof(stats));
       for(i=0;i<l;i++)
-        stats[(unsigned char)buffer_lower[i]]++;
+	stats[(unsigned char)buffer_lower[i]]++;
       ind=0;
       for(i=0;i<256;i++)
-        if(stats[i]>0)
-          ind+=stats[i]*(stats[i]-1);
+	if(stats[i]>0)
+	  ind+=stats[i]*(stats[i]-1);
       ind=ind/l/(l-1);
     }
     /* Detect .ini */
@@ -1163,85 +1213,85 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       ext="json";
     else
       ext=file_hint_txt.extension;
-    if(ext!=NULL && strcmp(ext,"txt")==0 &&
-        (strstr(buffer_lower,"<br>")!=NULL || strstr(buffer_lower,"<p>")!=NULL))
+    if(ext==NULL)
+      return 0;
+    if(strcmp(ext,"txt")==0 &&
+	(strstr(buffer_lower,"<br>")!=NULL || strstr(buffer_lower,"<p>")!=NULL))
     {
       ext="html";
     }
-    if(ext!=NULL && file_recovery!=NULL && file_recovery->file_stat!=NULL)
+    if(file_recovery!=NULL && file_recovery->file_stat!=NULL)
     {
       const unsigned char zip_header[4]  = { 'P', 'K', 0x03, 0x04};
       if(strcmp(ext,"html")==0 &&
-          file_recovery->file_stat->file_hint==&file_hint_txt &&
-          strstr(file_recovery->filename,"")!=NULL)
+	  file_recovery->file_stat->file_hint==&file_hint_txt &&
+	  strstr(file_recovery->filename,"")!=NULL)
       {
-        return 0;
+	return 0;
       }
 
       /* Special case: doc, texte files
 Unix: \n (0xA)
 Dos: \r\n (0xD 0xA)
 Doc: \r (0xD)
-       */
+*/
       if(file_recovery->file_stat->file_hint==&file_hint_doc &&
-          strstr(file_recovery->filename,".doc")!=NULL)
+	  strstr(file_recovery->filename,".doc")!=NULL)
       {
-        unsigned int i;
-        unsigned int txt_nl=0;
-        for(i=0;i<l-1;i++)
-          if(buffer_lower[i]=='\r' && buffer_lower[i+1]!='\n')
-          {
-            return 0;
-          }
-        for(i=0;i<l && i<512;i++)
-          if(buffer_lower[i]=='\n')
-            txt_nl=1;
-        if(txt_nl==1)
-        {
-          /* log_trace(">%s<\ndoc => %s\n",buffer_lower,ext); */
-          reset_file_recovery(file_recovery_new);
-          file_recovery_new->data_check=&data_check_txt;
-          file_recovery_new->file_check=&file_check_size;
-          file_recovery_new->extension=ext;
-          return 1;
-        }
+	unsigned int i;
+	unsigned int txt_nl=0;
+	if(ind>0.20)
+	  return 0;
+	for(i=0;i<l-1;i++)
+	  if(buffer_lower[i]=='\r' && buffer_lower[i+1]!='\n')
+	  {
+	    return 0;
+	  }
+	for(i=0;i<l && i<512;i++)
+	  if(buffer_lower[i]=='\n')
+	    txt_nl++;
+	if(txt_nl==0)
+	  return 0;
+	/* log_trace(">%s<\ndoc => %s\n",buffer_lower,ext); */
+	reset_file_recovery(file_recovery_new);
+	file_recovery_new->data_check=&data_check_txt;
+	file_recovery_new->file_check=&file_check_size;
+	file_recovery_new->extension=ext;
+	return 1;
       }
       buffer_lower[511]='\0';
       /* Special case: two consecutive HTML files */
       if((strcmp(ext,"html")==0 &&
-            strstr(buffer_lower,sign_html)!=NULL &&
-            strstr(file_recovery->filename,".html")!=NULL) ||
-          /* Text should not be found in JPEG */
-          (file_recovery->file_stat->file_hint==&file_hint_jpg &&
-           td_memmem(buffer, buffer_size_test, "8BIM", 4)==NULL &&
-           td_memmem(buffer, buffer_size_test, "adobe", 5)==NULL &&
-           td_memmem(buffer, buffer_size_test, "exif:", 5)==NULL &&
-           td_memmem(buffer, buffer_size_test, "<rdf:", 5)==NULL &&
-           td_memmem(buffer, buffer_size_test, "<?xpacket", 9)==NULL &&
-           td_memmem(buffer, buffer_size_test, "<dict>", 6)==NULL) ||
-          /* Text should not be found in zip because of compression */
-          (file_recovery->file_stat->file_hint==&file_hint_zip &&
-	  td_memmem(buffer, buffer_size_test, zip_header, 4)==NULL))
+	    strstr(buffer_lower,sign_html)!=NULL &&
+	    strstr(file_recovery->filename,".html")!=NULL) ||
+	  /* Text should not be found in JPEG */
+	  (file_recovery->file_stat->file_hint==&file_hint_jpg &&
+	   td_memmem(buffer, buffer_size_test, "8BIM", 4)==NULL &&
+	   td_memmem(buffer, buffer_size_test, "adobe", 5)==NULL &&
+	   td_memmem(buffer, buffer_size_test, "exif:", 5)==NULL &&
+	   td_memmem(buffer, buffer_size_test, "<rdf:", 5)==NULL &&
+	   td_memmem(buffer, buffer_size_test, "<?xpacket", 9)==NULL &&
+	   td_memmem(buffer, buffer_size_test, "<dict>", 6)==NULL) ||
+	  /* Text should not be found in zip because of compression except .sh3d */
+	  (file_recovery->file_stat->file_hint==&file_hint_zip &&
+	   td_memmem(buffer, buffer_size_test, zip_header, 4)==NULL &&
+	   strstr(file_recovery->filename, ".sh3d")==NULL))
       {
-        reset_file_recovery(file_recovery_new);
-        file_recovery_new->data_check=&data_check_txt;
-        file_recovery_new->file_check=&file_check_size;
-        file_recovery_new->extension=ext;
-        return 1;
+	reset_file_recovery(file_recovery_new);
+	file_recovery_new->data_check=&data_check_txt;
+	file_recovery_new->file_check=&file_check_size;
+	file_recovery_new->extension=ext;
+	return 1;
       }
       return 0;
     }
     /*    log_trace("ext=%s\n",ext); */
-    if(ext!=NULL)
-    {
-      reset_file_recovery(file_recovery_new);
-      file_recovery_new->extension=ext;
-      file_recovery_new->data_check=&data_check_txt;
-      file_recovery_new->file_check=&file_check_size;
-      return 1;
-    }
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->extension=ext;
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    return 1;
   }
-  return 0;
 }
 
 static int data_check_txt(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
