@@ -41,6 +41,7 @@
 #include "log.h"
 
 /* #define DEBUG_ZIP */
+extern const file_hint_t file_hint_doc;
 static void register_header_check_zip(file_stat_t *file_stat);
 static int header_check_zip(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 static void file_check_zip(file_recovery_t *file_recovery);
@@ -138,7 +139,6 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
   zip_file_entry_t  file;
   uint32_t          len;
 
-  fr->file_size += sizeof(file);
   if (fread(&file, sizeof(file), 1, fr->handle) != 1)
   {
 #ifdef DEBUG_ZIP
@@ -146,6 +146,7 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 #endif
     return -1;
   }
+  fr->file_size += sizeof(file);
 #ifdef DEBUG_ZIP
   log_info("%u Comp=%u %u CRC32=0x%08X ",
       le32(file.compressed_size),
@@ -157,7 +158,6 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
   if (len)
   {
     char *filename=(char *)MALLOC(len+1);
-    fr->file_size += len;
     if (fread(filename, len, 1, fr->handle) != 1)
     {
 #ifdef DEBUG_ZIP
@@ -166,6 +166,7 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
       free(filename);
       return -1;
     }
+    fr->file_size += len;
     filename[len]='\0';
     if(first_filename[0]=='\0')
     {
@@ -179,27 +180,61 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
     if(*ext==NULL)
     {
       static int msoffice=0;
+      static int sh3d=0;
       if(file_nbr==0)
       {
 	msoffice=0;
-	if(len==36 && memcmp(filename, "mimetypeapplication/vnd.sun.xml.calc", 36)==0)
-	  *ext="sxc";
-	else if(len==36 && memcmp(filename, "mimetypeapplication/vnd.sun.xml.draw", 36)==0)
-	  *ext="sxd";
-	else if(len==39 && memcmp(filename, "mimetypeapplication/vnd.sun.xml.impress", 39)==0)
-	  *ext="sxi";
-	else if(len==38 && memcmp(filename, "mimetypeapplication/vnd.sun.xml.writer", 38)==0)
-	  *ext="sxw";
-	else if(len==51 && memcmp(filename, "mimetypeapplication/vnd.oasis.opendocument.graphics",51)==0)
-	  *ext="odg";
-	else if(len==55 && memcmp(filename, "mimetypeapplication/vnd.oasis.opendocument.presentation",55)==0)
-	  *ext="odp";
-	else if(len==54 && memcmp(filename, "mimetypeapplication/vnd.oasis.opendocument.spreadsheet",54)==0)
-	  *ext="ods";
-	else if(len==47 && memcmp(filename, "mimetypeapplication/vnd.oasis.opendocument.text",47)==0)
-	  *ext="odt";
+	sh3d=0;
+	if(len==8 && memcmp(filename, "mimetype", 8)==0 &&
+	    le16(file.extra_length)==0 &&
+	    le32(file.compressed_size)==le32(file.uncompressed_size) &&
+	    le32(file.compressed_size)<=128)
+	{
+	  const int compressed_size=le32(file.uncompressed_size);
+	  unsigned char buffer[128];
+	  if( fread(buffer, compressed_size, 1, fr->handle)!=1)
+	  {
+#ifdef DEBUG_ZIP
+	    log_trace("zip: Unexpected EOF in file_entry data: %u bytes expected\n",
+	    compressed_size);
+#endif
+	    return -1;
+	  }
+	  if (fseek(fr->handle, -compressed_size, SEEK_CUR) < 0)
+	  {
+	    log_info("fseek failed CGR\n");
+	    return -1;
+	  }
+	  if(compressed_size==28 && memcmp(buffer,"application/vnd.sun.xml.calc",28)==0)
+	    *ext="sxc";
+	  else if(compressed_size==28 && memcmp(buffer,"application/vnd.sun.xml.draw",28)==0)
+	    *ext="sxd";
+	  else if(compressed_size==31 && memcmp(buffer,"application/vnd.sun.xml.impress",31)==0)
+	    *ext="sxi";
+	  else if(compressed_size==30 && memcmp(buffer,"application/vnd.sun.xml.writer",30)==0)
+	    *ext="sxw";
+	  else if(compressed_size==39 && memcmp(buffer,"application/vnd.oasis.opendocument.text",39)==0)
+	    *ext="odt";
+	  else if(compressed_size==43 && memcmp(buffer,"application/vnd.oasis.opendocument.graphics",43)==0)
+	    *ext="odg";
+	  else if(compressed_size==46 && memcmp(buffer,"application/vnd.oasis.opendocument.spreadsheet",46)==0)
+	    *ext="ods";
+	  else if(compressed_size==47 && memcmp(buffer,"application/vnd.oasis.opendocument.presentation",47)==0)
+	    *ext="odp";
+	  else
+	  { /* default to writer */
+	    *ext="sxw";
+	  }
+	}
 	else if(len==19 && memcmp(filename, "[Content_Types].xml", 19)==0)
 	  msoffice=1;
+	else if(len==4 && memcmp(filename, "Home", 4)==0)
+	  sh3d=1;
+      }
+      else if(file_nbr==1 && sh3d==1)
+      {
+	if(len==1 && filename[0]=='0')
+	  *ext="sh3d";
       }
       else if(file_nbr==2 && msoffice!=0)
       {
@@ -226,7 +261,6 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
   len = le16(file.extra_length);
   if (len>0)
   {
-    fr->file_size += len;
     if (fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
@@ -234,11 +268,11 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 #endif
       return -1;
     }
+    fr->file_size += len;
   }
   len = le32(file.compressed_size);
   if (len>0)
   {
-    fr->file_size += len;
     if (fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
@@ -249,6 +283,7 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 #ifdef DEBUG_ZIP
     log_trace("zip: Data of length %u\n", len);
 #endif
+    fr->file_size += len;
   }
 
   expected_compressed_size=0;
@@ -265,6 +300,8 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 #ifdef DEBUG_ZIP
     log_trace("Searched footer, got length %lli\n", (long long int)pos);
 #endif
+    if (pos < 0)
+      return -1;
     if (pos > 0)
     {
       fr->file_size += pos;
@@ -287,7 +324,6 @@ static int zip_parse_central_dir(file_recovery_t *fr)
     uint32_t offset_header;           /** Relative offset of local header */
   } __attribute__ ((__packed__)) dir;
 
-  fr->file_size += 2;
   if (fseek(fr->handle, 2, SEEK_CUR) == -1)
   {
 #ifdef DEBUG_ZIP
@@ -295,8 +331,8 @@ static int zip_parse_central_dir(file_recovery_t *fr)
 #endif
     return -1;
   }
+  fr->file_size += 2;
 
-  fr->file_size += sizeof(file);
   if (fread(&file, sizeof(file), 1, fr->handle) != 1)
   {
 #ifdef DEBUG_ZIP
@@ -304,11 +340,11 @@ static int zip_parse_central_dir(file_recovery_t *fr)
 #endif
     return -1;
   }
+  fr->file_size += sizeof(file);
 #ifdef DEBUG_ZIP
   log_trace("zip: Central dir with CRC 0x%08X\n", file.crc32);
 #endif
 
-  fr->file_size += sizeof(dir);
   if (fread(&dir, sizeof(dir), 1, fr->handle) != 1)
   {
 #ifdef DEBUG_ZIP
@@ -316,10 +352,10 @@ static int zip_parse_central_dir(file_recovery_t *fr)
 #endif
     return -1;
   }
+  fr->file_size += sizeof(dir);
 
   /* Rest of the block - could attempt CRC check */
   len = le16(file.extra_length) + le16(dir.comment_length) + le16(file.filename_length);
-  fr->file_size += len;
   if (fseek(fr->handle, len, SEEK_CUR) == -1)
   {
 #ifdef DEBUG_ZIP
@@ -327,6 +363,7 @@ static int zip_parse_central_dir(file_recovery_t *fr)
 #endif
     return -1;
   }
+  fr->file_size += len;
 #ifdef DEBUG_ZIP
   log_trace("zip: Data of total length %u\n", len);
 #endif
@@ -347,7 +384,6 @@ static int zip64_parse_end_central_dir(file_recovery_t *fr)
     uint64_t offset;                  /** Offset of start of central directory */
   } __attribute__ ((__packed__)) dir;
 
-  fr->file_size += sizeof(dir);
   if (fread(&dir, sizeof(dir), 1, fr->handle) != 1)
   {
 #ifdef DEBUG_ZIP
@@ -355,11 +391,11 @@ static int zip64_parse_end_central_dir(file_recovery_t *fr)
 #endif
     return -1;
   }
+  fr->file_size += sizeof(dir);
 
   if (dir.end_size > 0)
   {
     uint64_t len = le64(dir.end_size);
-    fr->file_size += len;
     if (fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
@@ -367,6 +403,7 @@ static int zip64_parse_end_central_dir(file_recovery_t *fr)
 #endif
       return -1;
     }
+    fr->file_size += len;
 #ifdef DEBUG_ZIP
     log_trace("zip: End of 64b central dir of length %llu\n", (long long unsigned)len);
 #endif
@@ -387,7 +424,6 @@ static int zip_parse_end_central_dir(file_recovery_t *fr)
     uint16_t comment_length;          /** Comment length */
   } __attribute__ ((__packed__)) dir;
 
-  fr->file_size += sizeof(dir);
   if (fread(&dir, sizeof(dir), 1, fr->handle) != 1)
   {
 #ifdef DEBUG_ZIP
@@ -395,11 +431,11 @@ static int zip_parse_end_central_dir(file_recovery_t *fr)
 #endif
     return -1;
   }
+  fr->file_size += sizeof(dir);
 
   if (dir.comment_length)
   {
     uint16_t len = le16(dir.comment_length);
-    fr->file_size += len;
     if (fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
@@ -407,6 +443,7 @@ static int zip_parse_end_central_dir(file_recovery_t *fr)
 #endif
       return -1;
     }
+    fr->file_size += len;
 #ifdef DEBUG_ZIP
     log_trace("zip: Comment of length %u\n", len);
 #endif
@@ -422,7 +459,6 @@ static int zip_parse_data_desc(file_recovery_t *fr)
     uint32_t uncompressed_size;      /** Uncompressed size (bytes) */
   } __attribute__ ((__packed__)) desc;
 
-  fr->file_size += sizeof(desc);
   if (fread(&desc, sizeof(desc), 1, fr->handle) != 1)
   {
 #ifdef DEBUG_ZIP
@@ -430,6 +466,7 @@ static int zip_parse_data_desc(file_recovery_t *fr)
 #endif
     return -1;
   }
+  fr->file_size += sizeof(desc);
 #ifdef DEBUG_ZIP
   log_info("%u %u CRC32=0x%08X\n",
       le32(desc.compressed_size),
@@ -445,7 +482,6 @@ static int zip_parse_signature(file_recovery_t *fr)
 {
   uint16_t len;
 
-  fr->file_size += 2;
   if (fread(&len, 2, 1, fr->handle) != 1)
   {
 #ifdef DEBUG_ZIP
@@ -453,11 +489,11 @@ static int zip_parse_signature(file_recovery_t *fr)
 #endif
     return -1;
   }
+  fr->file_size += 2;
 
   if (len)
   {
     len = le16(len);
-    fr->file_size += len;
     if (fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
@@ -465,6 +501,7 @@ static int zip_parse_signature(file_recovery_t *fr)
 #endif
       return -1;
     }
+    fr->file_size += len;
   }
 
   return 0;
@@ -478,7 +515,6 @@ static int zip64_parse_end_central_dir_locator(file_recovery_t *fr)
     uint32_t disk_total_number; /** Total number of disks */
   } __attribute__ ((__packed__)) loc;
 
-  fr->file_size += sizeof(loc);
   if (fread(&loc, sizeof(loc), 1, fr->handle) != 1)
   {
 #ifdef DEBUG_ZIP
@@ -486,6 +522,7 @@ static int zip64_parse_end_central_dir_locator(file_recovery_t *fr)
 #endif
     return -1;
   }
+  fr->file_size += sizeof(loc);
   return 0;
 }
 
@@ -493,11 +530,12 @@ static void file_check_zip(file_recovery_t *fr)
 {
   const char *ext=NULL;
   unsigned int file_nbr=0;
-  fseek(fr->handle, 0, SEEK_SET);
   fr->file_size = 0;
   fr->offset_error=0;
   fr->offset_ok=0;
   first_filename[0]='\0';
+  if(fseek(fr->handle, 0, SEEK_SET) < 0)
+    return ;
   while (1)
   {
     uint64_t file_size_old;
@@ -579,11 +617,14 @@ static void file_rename_zip(const char *old_filename)
   reset_file_recovery(&fr);
   if((fr.handle=fopen(old_filename, "rb"))==NULL)
     return;
-  fseek(fr.handle, 0, SEEK_SET);
   fr.file_size = 0;
   fr.offset_error=0;
   first_filename[0]='\0';
-
+  if(fseek(fr.handle, 0, SEEK_SET) < 0)
+  {
+    fclose(fr.handle);
+    return ;
+  }
   while (1)
   {
     uint32_t header;
@@ -682,55 +723,58 @@ static int header_check_zip(const unsigned char *buffer, const unsigned int buff
 #endif
   if(memcmp(buffer,zip_header,sizeof(zip_header))==0)
   {
+    const zip_file_entry_t *file=(const zip_file_entry_t *)&buffer[4];
+    const unsigned int len=le16(file->filename_length);
+    if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+	file_recovery->file_stat->file_hint==&file_hint_doc &&
+	(strcmp(file_recovery->extension,"doc")==0 ||
+	 strcmp(file_recovery->extension,"psmodel")==0)
+	&& memcmp(&buffer[30], "macrolog_1.mac", 14)==0)
+      return 0;
     reset_file_recovery(file_recovery_new);
     file_recovery_new->min_filesize=21;
     file_recovery_new->file_check=&file_check_zip;
-    if(memcmp(&buffer[30],"mimetypeapplication/vnd.sun.xml.",32)==0)
-     {
-       if(memcmp(&buffer[62],"calc",4)==0)
-       {
-         file_recovery_new->extension="sxc";
-       }
-       else if(memcmp(&buffer[62],"draw",4)==0)
-       {
-         file_recovery_new->extension="sxd";
-       }
-       else if(memcmp(&buffer[62],"impress",7)==0)
-       {
-         file_recovery_new->extension="sxi";
-       }
-       else if(memcmp(&buffer[62],"writer",6)==0)
-       {
-         file_recovery_new->extension="sxw";
-       }
-       else
-       { /* default to writer */
-         file_recovery_new->extension="sxw";
-       }
-     }
-     else if(memcmp(&buffer[30],"mimetypeapplication/vnd.oasis.opendocument.graphics",51)==0)
-       file_recovery_new->extension="odg";
-     else if(memcmp(&buffer[30],"mimetypeapplication/vnd.oasis.opendocument.presentation",55)==0)
-       file_recovery_new->extension="odp";
-     else if(memcmp(&buffer[30],"mimetypeapplication/vnd.oasis.opendocument.spreadsheet",54)==0)
-       file_recovery_new->extension="ods";
-     else if(memcmp(&buffer[30],"mimetypeapplication/vnd.oasis.opendocument.text",47)==0)
-       file_recovery_new->extension="odt";
-     else if(memcmp(&buffer[30],"[Content_Types].xml",19)==0)
-     {
-       if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"word/", 5)!=0)
-         file_recovery_new->extension="docx";
-       else if(pos_in_mem(&buffer[0], 2000, (const unsigned char*)"xl/", 3)!=0)
-         file_recovery_new->extension="xlsx";
-       else if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"ppt/", 4)!=0)
-         file_recovery_new->extension="pptx";
-       else
-         file_recovery_new->extension="docx";
-       file_recovery_new->file_rename=&file_rename_zip;
-     }
-     /* Extended Renoise song file */
-    else if(memcmp(&buffer[30], "Song.xml", 8)==0)
+    if(len==8 && memcmp(&buffer[30],"mimetype",8)==0)
+    {
+      const unsigned int compressed_size=le32(file->uncompressed_size);
+      if(compressed_size==28 && memcmp(&buffer[38],"application/vnd.sun.xml.calc",28)==0)
+	file_recovery_new->extension="sxc";
+      else if(compressed_size==28 && memcmp(&buffer[38],"application/vnd.sun.xml.draw",28)==0)
+	file_recovery_new->extension="sxd";
+      else if(compressed_size==31 && memcmp(&buffer[38],"application/vnd.sun.xml.impress",31)==0)
+	file_recovery_new->extension="sxi";
+      else if(compressed_size==30 && memcmp(&buffer[38],"application/vnd.sun.xml.writer",30)==0)
+	file_recovery_new->extension="sxw";
+      else if(compressed_size==39 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.text",39)==0)
+	file_recovery_new->extension="odt";
+      else if(compressed_size==43 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.graphics",43)==0)
+	file_recovery_new->extension="odg";
+      else if(compressed_size==46 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.spreadsheet",46)==0)
+	file_recovery_new->extension="ods";
+      else if(compressed_size==47 && memcmp(&buffer[38],"application/vnd.oasis.opendocument.presentation",47)==0)
+	file_recovery_new->extension="odp";
+      else
+      { /* default to writer */
+	file_recovery_new->extension="sxw";
+      }
+    }
+    else if(len==19 && memcmp(&buffer[30],"[Content_Types].xml",19)==0)
+    {
+      if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"word/", 5)!=0)
+	file_recovery_new->extension="docx";
+      else if(pos_in_mem(&buffer[0], 2000, (const unsigned char*)"xl/", 3)!=0)
+	file_recovery_new->extension="xlsx";
+      else if(pos_in_mem(&buffer[0], buffer_size, (const unsigned char*)"ppt/", 4)!=0)
+	file_recovery_new->extension="pptx";
+      else
+	file_recovery_new->extension="docx";
+      file_recovery_new->file_rename=&file_rename_zip;
+    }
+    /* Extended Renoise song file */
+    else if(len==8 && memcmp(&buffer[30], "Song.xml", 8)==0)
       file_recovery_new->extension="xrns";
+    else if(len==4 && memcmp(&buffer[30], "Home", 4)==0)
+      file_recovery_new->extension="sh3d";
     else
     {
       file_recovery_new->extension=file_hint_zip.extension;
