@@ -188,40 +188,44 @@ void file_allow_nl(file_recovery_t *file_recovery, const unsigned int nl_mode)
     file_recovery->file_size++;
 }
 
-void file_search_footer(file_recovery_t *file_recovery, const void*footer, const unsigned int footer_length, const unsigned int extra_length)
+uint64_t file_rsearch(FILE *handle, uint64_t offset, const void*footer, const unsigned int footer_length)
 {
   unsigned char*buffer;
-  int64_t file_size;
-  if(footer_length==0 || file_recovery->file_size <= extra_length)
-    return ;
   buffer=(unsigned char*)MALLOC(4096+footer_length-1);
-  file_size=file_recovery->file_size-extra_length;
   memset(buffer+4096,0,footer_length-1);
   do
   {
     int i;
     int taille;
-    const unsigned int read_size=(file_size%4096!=0 ? file_size%4096 : 4096);
-    file_size-=read_size;
-    if(fseek(file_recovery->handle,file_size,SEEK_SET)<0)
+    const unsigned int read_size=(offset%4096!=0 ? offset%4096 : 4096);
+    offset-=read_size;
+    if(fseek(handle,offset,SEEK_SET)<0)
     {
       free(buffer);
-      return;
+      return 0;
     }
-    taille=fread(buffer,1,read_size,file_recovery->handle);
+    taille=fread(buffer, 1, read_size, handle);
     for(i=taille-1;i>=0;i--)
     {
       if(buffer[i]==*(const unsigned char *)footer && memcmp(buffer+i,footer,footer_length)==0)
       {
-        file_recovery->file_size=file_size + i + footer_length + extra_length;
         free(buffer);
-        return;
+        return offset + i;
       }
     }
     memcpy(buffer+read_size,buffer,footer_length-1);
-  } while(file_size>0);
-  file_recovery->file_size=0;
+  } while(offset>0);
   free(buffer);
+  return 0;
+}
+
+void file_search_footer(file_recovery_t *file_recovery, const void*footer, const unsigned int footer_length, const unsigned int extra_length)
+{
+  if(footer_length==0 || file_recovery->file_size <= extra_length)
+    return ;
+  file_recovery->file_size=file_rsearch(file_recovery->handle, file_recovery->file_size-extra_length, footer, footer_length);
+  if(file_recovery->file_size > 0)
+    file_recovery->file_size+= footer_length + extra_length;
 }
 
 void file_search_lc_footer(file_recovery_t *file_recovery, const unsigned char*footer, const unsigned int footer_length)
@@ -360,6 +364,8 @@ void file_rename(const char *old_filename, const unsigned char *buffer, const in
   char *dst;
   char *directory_sep;
   int len=strlen(old_filename)+1;
+  if(buffer_size<0)
+    return ;
   if(offset < buffer_size && buffer!=NULL)
     len+=buffer_size-offset+1;
   if(new_ext!=NULL)
@@ -382,7 +388,10 @@ void file_rename(const char *old_filename, const unsigned char *buffer, const in
   /* Add original filename */
   if(offset < buffer_size && buffer!=NULL)
   {
+    char *dst_old=dst;
     int off;
+    int ok=0;
+    int bad=0;
     *dst++ = '_';
     for(off=offset; off<buffer_size && buffer[off]!='\0'; off++)
     {
@@ -391,16 +400,31 @@ void file_rename(const char *old_filename, const unsigned char *buffer, const in
 	case '/':
 	case '\\':
 	case ':':
+	case '*':
 	  *dst++ = '_';
+	  bad++;
 	  break;
 	default:
 	  if(isprint(buffer[off]) && !isspace(buffer[off]))
+	  {
 	    *dst++ = buffer[off];
+	    ok++;
+	  }
 	  else
+	  {
 	    *dst++ = '_';
+	    bad++;
+	  }
 	  break;
       }
     }
+    if(ok <= bad)
+      dst=dst_old;
+    else
+    {
+      while(dst > dst_old && *(dst-1)=='_')
+	dst--;
+    }
   }
   /* Add extension */
   if(new_ext!=NULL)
@@ -435,6 +459,8 @@ void file_rename_unicode(const char *old_filename, const unsigned char *buffer,
   char *dst;
   char *directory_sep;
   int len=strlen(old_filename)+1;
+  if(buffer_size<0)
+    return ;
   if(offset < buffer_size && buffer!=NULL)
     len+=buffer_size-offset;
   if(new_ext!=NULL)
@@ -457,7 +483,10 @@ void file_rename_unicode(const char *old_filename, const unsigned char *buffer,
   /* Add original filename */
   if(offset < buffer_size && buffer!=NULL)
   {
+    char *dst_old=dst;
     int off;
+    int ok=0;
+    int bad=0;
     *dst++ = '_';
     for(off=offset; off<buffer_size && buffer[off]!='\0'; off+=2)
     {
@@ -466,16 +495,31 @@ void file_rename_unicode(const char *old_filename, const unsigned char *buffer,
 	case '/':
 	case '\\':
 	case ':':
+	case '*':
 	  *dst++ = '_';
+	  bad++;
 	  break;
 	default:
 	  if(isprint(buffer[off]) && !isspace(buffer[off]))
+	  {
 	    *dst++ = buffer[off];
+	    ok++;
+	  }
 	  else
+	  {
 	    *dst++ = '_';
+	    bad++;
+	  }
 	  break;
       }
     }
+    if(ok <= bad)
+      dst=dst_old;
+    else
+    {
+      while(dst > dst_old && *(dst-1)=='_')
+	dst--;
+    }
   }
   /* Add extension */
   if(new_ext!=NULL)
