@@ -22,7 +22,13 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #include <stdio.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 #include <stdarg.h>
 #ifdef HAVE_TIME_H
 #include <time.h>
@@ -33,11 +39,12 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_SYS_CYGWIN_H
+#include <sys/cygwin.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "log.h"
-#include "intrf.h"	/* ask_log_location */
-#include "fnctdsk.h"
 
 static FILE *log_handle=NULL;
 static int f_status=0;
@@ -52,43 +59,69 @@ int log_set_levels(const unsigned int levels)
   return old_levels;
 }
 
-int log_open(const char*default_filename, const int mode, const int ncurses_interface, const char *prog_name, int argc, char**argv)
+FILE *log_open(const char*default_filename, const int mode)
 {
-  const char*filename=default_filename;
-  if(mode!=TD_LOG_CREATE && mode!=TD_LOG_APPEND)
-    return mode;
-  do
+  log_handle=fopen(default_filename,(mode==TD_LOG_CREATE?"w":"a"));
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+  if(log_handle!=NULL && mode!=TD_LOG_CREATE)
   {
-    log_handle=fopen(filename,(mode==TD_LOG_CREATE?"w":"a"));
-    if(log_handle==NULL)
+    /* append doesn't work when running the executable via wine */
+    if(fprintf(log_handle, "\n")<=0 || fflush(log_handle)!=0)
     {
-      if(ncurses_interface==0)
-      {
-	printf("Can't create %s file\n", default_filename);
-	return mode;
-      }
-      filename=ask_log_location(filename);
-      if(filename==NULL)
-	return TD_LOG_REFUSED;
+      fclose(log_handle);
+      log_handle=fopen(default_filename,"w");
     }
-  } while(log_handle==NULL);
-  {
-    int i;
-    time_t my_time;
-#ifdef HAVE_DUP2
-    dup2(fileno(log_handle),2);
-#endif
-    my_time=time(NULL);
-    fprintf(log_handle,"\n\n%s",ctime(&my_time));
-    fprintf(log_handle,"Command line: %s", prog_name);
-    for(i=1;i<argc;i++)
-      fprintf(log_handle," %s", argv[i]);
-    fprintf(log_handle,"\n\n");
-    fflush(log_handle);
   }
-  return TD_LOG_DONE;
+#endif
+  return log_handle;
 }
 
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+FILE *log_open_default(const char*default_filename, const int mode)
+{
+  char*filename;
+  char *path;
+  if(log_handle != NULL)
+    return log_handle;
+  path = getenv("USERPROFILE");
+  if (path == NULL)
+    path = getenv("HOMEPATH");
+  if(path == NULL)
+    return log_open(default_filename, mode);
+  /* Check to avoid buffer overflow may not be 100% bullet proof */
+  if(strlen(path)+strlen(default_filename)+2 > 4096)
+    return log_open(default_filename, mode);
+  filename=(char*)MALLOC(4096);
+#ifdef __CYGWIN__
+/* FIXME */
+  cygwin_conv_to_posix_path(path, filename);
+#else
+  strcpy(filename, path);
+#endif
+  strcat(filename, "/");
+  strcat(filename, default_filename);
+  log_open(filename, mode);
+  free(filename);
+  return log_handle;
+}
+#else
+FILE *log_open_default(const char*default_filename, const int mode)
+{
+  char*filename;
+  char *path;
+  path = getenv("HOME");
+  if(path == NULL)
+    return log_open(default_filename, mode);
+  filename=(char*)MALLOC(strlen(path)+strlen(default_filename)+2);
+  strcpy(filename, path);
+  strcat(filename, "/");
+  strcat(filename, default_filename);
+  log_open(default_filename, mode);
+  free(filename);
+  return log_handle;
+}
+#endif
+
 int log_flush(void)
 {
   return fflush(log_handle);
@@ -119,6 +152,7 @@ int log_close(void)
   {
     if(fclose(log_handle))
       f_status=1;
+    log_handle=NULL;
   }
   return f_status;
 }
@@ -244,15 +278,3 @@ void dump2_log(const void *dump_1, const void *dump_2, const unsigned int lng)
     log_info("\n");
   }
 }
-
-void log_partition(const disk_t *disk_car,const partition_t *partition)
-{
-  const char *msg;
-  char buffer_part_size[100];
-  msg=aff_part_aux(AFF_PART_ORDER|AFF_PART_STATUS, disk_car, partition);
-  log_info("%s",msg);
-  if(partition->info[0]!='\0')
-    log_info("\n     %s, %s",partition->info,size_to_unit(partition->part_size,buffer_part_size));
-  log_info("\n");
-}
-
