@@ -2,7 +2,7 @@
 
     File: ntfs.c
 
-    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2008 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include <config.h>
 #endif
  
+#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -44,10 +45,10 @@
 #include "log.h"
 /* #include "guid_cmp.h" */
 extern const arch_fnct_t arch_i386;
-extern const arch_fnct_t arch_mac;
 
-static int ntfs_read_MFT(disk_t *disk_car, partition_t *partition, const struct ntfs_boot_sector*ntfs_header,const int my_type,const int verbose, const int dump_ind);
-static int ntfs_get_attr_aux(const char *attr_record, const int my_type, partition_t *partition, const char *end, const int verbose, const int dump_ind, const char*file_name_to_find);
+static int set_NTFS_info(disk_t *disk_car, const struct ntfs_boot_sector*ntfs_header,partition_t *partition,const int verbose);
+static int ntfs_read_MFT(disk_t *disk_car, partition_t *partition, const struct ntfs_boot_sector*ntfs_header, const int my_type, const int verbose);
+static int ntfs_get_attr_aux(const char *attr_record, const int my_type, partition_t *partition, const char *end, const int verbose, const char*file_name_to_find);
 
 unsigned int ntfs_sector_size(const struct ntfs_boot_sector *ntfs_header)
 { return (ntfs_header->sector_size[1]<<8)+ntfs_header->sector_size[0]; }
@@ -56,7 +57,7 @@ int check_NTFS(disk_t *disk_car,partition_t *partition,const int verbose,const i
 {
   unsigned char *buffer=(unsigned char*)MALLOC(DEFAULT_SECTOR_SIZE);
 /*  log_trace("check_NTFS part_offset=%llu\n",(long long unsigned)partition->part_offset); */
-  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
+  if(disk_car->pread(disk_car, buffer, DEFAULT_SECTOR_SIZE, partition->part_offset) != DEFAULT_SECTOR_SIZE)
   {
     free(buffer);
     return 1;
@@ -66,7 +67,7 @@ int check_NTFS(disk_t *disk_car,partition_t *partition,const int verbose,const i
     free(buffer);
     return 1;
   }
-  set_NTFS_info(disk_car,(struct ntfs_boot_sector*)buffer,partition,verbose,dump_ind);
+  set_NTFS_info(disk_car, (struct ntfs_boot_sector*)buffer, partition, verbose);
   free(buffer);
   return 0;
 }
@@ -105,18 +106,18 @@ int recover_NTFS(disk_t *disk_car, const struct ntfs_boot_sector*ntfs_header,par
   partition->part_size=part_size;
   partition->part_type_i386=P_NTFS;
   partition->part_type_gpt=GPT_ENT_TYPE_MS_BASIC_DATA;
-  set_NTFS_info(disk_car,ntfs_header,partition,verbose,dump_ind);
+  set_NTFS_info(disk_car, ntfs_header, partition, verbose);
   return 0;
 }
 
-int set_NTFS_info(disk_t *disk_car, const struct ntfs_boot_sector*ntfs_header,partition_t *partition,const int verbose, const int dump_ind)
+static int set_NTFS_info(disk_t *disk_car, const struct ntfs_boot_sector*ntfs_header,partition_t *partition,const int verbose)
 {
   partition->fsname[0]='\0';
   if(partition->sb_offset==0)
     strncpy(partition->info, "NTFS", sizeof(partition->info));
   else
     strncpy(partition->info, "NTFS found using backup sector!", sizeof(partition->info));
-  return ntfs_read_MFT(disk_car, partition, ntfs_header,0x60,verbose,dump_ind);
+  return ntfs_read_MFT(disk_car, partition, ntfs_header, 0x60, verbose);
 }
 
 int test_NTFS(const disk_t *disk_car,const struct ntfs_boot_sector*ntfs_header, partition_t *partition,const int verbose, const int dump_ind)
@@ -127,7 +128,8 @@ int test_NTFS(const disk_t *disk_car,const struct ntfs_boot_sector*ntfs_header,
       ntfs_header->dir_entries[0]!=0 || ntfs_header->dir_entries[1]!=0 ||
       ntfs_header->sectors[0]!=0 || ntfs_header->sectors[1]!=0 ||
       le16(ntfs_header->fat_length)!=0 || le32(ntfs_header->total_sect)!=0 ||
-      memcmp(ntfs_header->system_id,"NTFS",4)!=0)
+      memcmp(ntfs_header->system_id,"NTFS",4)!=0 ||
+      le64(ntfs_header->sectors_nbr)==0)
     return 1;
   switch(ntfs_header->sectors_per_cluster)
   {
@@ -140,20 +142,26 @@ int test_NTFS(const disk_t *disk_car,const struct ntfs_boot_sector*ntfs_header,
   {
     log_info("NTFS at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
   }
-  if(le16(ntfs_header->heads)!=disk_car->CHS.head+1)
+  if(le16(ntfs_header->heads)!=disk_car->geom.heads_per_cylinder)
   {
-    screen_buffer_add("Warning: Incorrect number of heads/cylinder %u (NTFS) != %u (HD)\n",le16(ntfs_header->heads),disk_car->CHS.head+1);
-    log_warning("heads/cylinder %u (NTFS) != %u (HD)\n",le16(ntfs_header->heads),disk_car->CHS.head+1);
+    screen_buffer_add("Warning: Incorrect number of heads/cylinder %u (NTFS) != %u (HD)\n",
+	le16(ntfs_header->heads), disk_car->geom.heads_per_cylinder);
+    log_warning("heads/cylinder %u (NTFS) != %u (HD)\n",
+	le16(ntfs_header->heads), disk_car->geom.heads_per_cylinder);
   }
-  if(le16(ntfs_header->secs_track)!=disk_car->CHS.sector)
+  if(le16(ntfs_header->secs_track)!=disk_car->geom.sectors_per_head)
   {
-    screen_buffer_add("Warning: Incorrect number of sectors per track %u (NTFS) != %u (HD)\n",le16(ntfs_header->secs_track),disk_car->CHS.sector);
-    log_warning("sect/track %u (NTFS) != %u (HD)\n",le16(ntfs_header->secs_track),disk_car->CHS.sector);
+    screen_buffer_add("Warning: Incorrect number of sectors per track %u (NTFS) != %u (HD)\n",
+	le16(ntfs_header->secs_track), disk_car->geom.sectors_per_head);
+    log_warning("sect/track %u (NTFS) != %u (HD)\n",
+	le16(ntfs_header->secs_track), disk_car->geom.sectors_per_head);
   }
   if(ntfs_sector_size(ntfs_header)!=disk_car->sector_size)
   {
-    screen_buffer_add("Warning: Incorrect number of bytes per sector %u (NTFS) != %u (HD)\n",ntfs_sector_size(ntfs_header),disk_car->sector_size);
-    log_warning("Warning: Incorrect number of bytes per sector %u (NTFS) != %u (HD)\n",ntfs_sector_size(ntfs_header),disk_car->sector_size);
+    screen_buffer_add("Warning: Incorrect number of bytes per sector %u (NTFS) != %u (HD)\n",
+	ntfs_sector_size(ntfs_header), disk_car->sector_size);
+    log_warning("Warning: Incorrect number of bytes per sector %u (NTFS) != %u (HD)\n",
+	ntfs_sector_size(ntfs_header), disk_car->sector_size);
   }
 
   if(partition->part_size>0)
@@ -177,7 +185,7 @@ int test_NTFS(const disk_t *disk_car,const struct ntfs_boot_sector*ntfs_header,
 }
 
 /* */
-int ntfs_get_attr(const char *mft_record, const int my_type, partition_t *partition, const char *end, const int verbose, const int dump_ind, const char*file_name_to_find)
+int ntfs_get_attr(const char *mft_record, const int my_type, partition_t *partition, const char *end, const int verbose, const char*file_name_to_find)
 {
   const char *attr_record;
   /* Only check for magic DWORD here, fixup should have happened before */
@@ -191,10 +199,10 @@ int ntfs_get_attr(const char *mft_record, const int my_type, partition_t *partit
   /*	screen_buffer_add("main MFT record %lu ",NTFS_GETU64(mft_record+0x20)); */
   /* location of first attribute */
   attr_record= mft_record + NTFS_GETU16(mft_record + 0x14);
-  return ntfs_get_attr_aux(attr_record, my_type, partition, end, verbose, dump_ind, file_name_to_find);
+  return ntfs_get_attr_aux(attr_record, my_type, partition, end, verbose, file_name_to_find);
 }
 
-static int ntfs_get_attr_aux(const char *attr_record, const int my_type, partition_t *partition, const char *end, const int verbose, const int dump_ind, const char*file_name_to_find)
+static int ntfs_get_attr_aux(const char *attr_record, const int my_type, partition_t *partition, const char *end, const int verbose, const char*file_name_to_find)
 {
   int attr_type;
   while(1)
@@ -302,7 +310,7 @@ static int ntfs_get_attr_aux(const char *attr_record, const int my_type, partiti
                 volume_name_length=sizeof(partition->fsname)-1;
               for(name_it=attr_td_list_entry;(volume_name_length>0) && (*name_it!='\0') && (name_it[1]=='\0'); name_it+=2,volume_name_length--)
                 *dest++=*name_it;
-              *dest++='\0'; /* 27 january 2003: Correct a bug found by Andreas du Plessis-Denz */
+              *dest='\0'; /* 27 january 2003: Correct a bug found by Andreas du Plessis-Denz */
             }
             return 1;
           case 0x90:	/* AT_INDEX_ROOT */
@@ -317,70 +325,68 @@ static int ntfs_get_attr_aux(const char *attr_record, const int my_type, partiti
         switch(attr_type)
         {
           case 0x80:	/* AT_DATA */
-            {
-              /* buf must be unsigned! */
-              const unsigned char *buf;
-              uint8_t b;                   	/* Current byte offset in buf. */
-              uint16_t mapping_pairs_offset;
-              const unsigned char*attr_end;     /* End of attribute. */
-              long lcn;
-              int64_t deltaxcn = (int64_t)-1;	/* Change in [vl]cn. */
-              mapping_pairs_offset=NTFS_GETU16(attr_record+32);
-              buf=(const unsigned char*)attr_record + mapping_pairs_offset;
-              attr_end = (const unsigned char*)attr_record + attr_len;
-              lcn = 0;
-              /* return first element of the run_list */
-              {
-                b = *buf & 0xf;
-                if (b){
-                  if (buf + b > attr_end)
-                  {
-                    log_error("Attribut AT_DATA: bad size\n");
-                    return 0;
-                  }
-                  for (deltaxcn = (int8_t)buf[b--]; b; b--)
-                    deltaxcn = (deltaxcn << 8) + (uint8_t)buf[b];
-                  /* Assume a negative length to indicate data corruption */
-                  if (deltaxcn < 0)
-                    log_error("Invalid length in mapping pairs array.\n");
-                } else { /* The length entry is compulsory. */
-                  log_error("Missing length entry in mapping pairs array.\n");
-                }
-                if (deltaxcn >= 0)
-                {
-                  if (!(*buf & 0xf0))
-                  {
-                    log_info("LCN_HOLE\n");
-                  }
-                  else
-                  {
-                    /* Get the lcn change which really can be negative. */
-                    uint8_t b2 = *buf & 0xf;
-                    b = b2 + ((*buf >> 4) & 0xf);
-                    if (buf + b > attr_end)
-                    {
-                      log_error("Attribut AT_DATA: bad size\n");
-                      return 0;
-                    }
-                    for (deltaxcn = (int8_t)buf[b--]; b > b2; b--)
-                      deltaxcn = (deltaxcn << 8) + (uint8_t)buf[b];
-                    /* Change the current lcn to it's new value. */
-                    lcn += deltaxcn;
-                    /* Check lcn is not below -1. */
-                    if (lcn < -1) {
-                      log_error("Invalid LCN < -1 in mapping pairs array.");
-                      return 0;
-                    }
-                    if(verbose>1)
-                    {
-                      log_verbose("LCN %ld\n",lcn);
-                    }
-                    if(attr_type==my_type)
-                      return lcn;
-                  }
-                }
-              }
-            }
+	    {
+	      /* buf must be unsigned! */
+	      const unsigned char *buf;
+	      uint8_t b;                   	/* Current byte offset in buf. */
+	      uint16_t mapping_pairs_offset;
+	      const unsigned char*attr_end;     /* End of attribute. */
+	      long lcn;
+	      int64_t deltaxcn = (int64_t)-1;	/* Change in [vl]cn. */
+	      mapping_pairs_offset=NTFS_GETU16(attr_record+32);
+	      buf=(const unsigned char*)attr_record + mapping_pairs_offset;
+	      attr_end = (const unsigned char*)attr_record + attr_len;
+	      lcn = 0;
+	      /* return first element of the run_list */
+	      b = *buf & 0xf;
+	      if (b){
+		if (buf + b > attr_end)
+		{
+		  log_error("Attribut AT_DATA: bad size\n");
+		  return 0;
+		}
+		for (deltaxcn = (int8_t)buf[b--]; b; b--)
+		  deltaxcn = (deltaxcn << 8) + (uint8_t)buf[b];
+		/* Assume a negative length to indicate data corruption */
+		if (deltaxcn < 0)
+		  log_error("Invalid length in mapping pairs array.\n");
+	      } else { /* The length entry is compulsory. */
+		log_error("Missing length entry in mapping pairs array.\n");
+	      }
+	      if (deltaxcn >= 0)
+	      {
+		if (!(*buf & 0xf0))
+		{
+		  log_info("LCN_HOLE\n");
+		}
+		else
+		{
+		  /* Get the lcn change which really can be negative. */
+		  uint8_t b2 = *buf & 0xf;
+		  b = b2 + ((*buf >> 4) & 0xf);
+		  if (buf + b > attr_end)
+		  {
+		    log_error("Attribut AT_DATA: bad size\n");
+		    return 0;
+		  }
+		  for (deltaxcn = (int8_t)buf[b--]; b > b2; b--)
+		    deltaxcn = (deltaxcn << 8) + (uint8_t)buf[b];
+		  /* Change the current lcn to it's new value. */
+		  lcn += deltaxcn;
+		  /* Check lcn is not below -1. */
+		  if (lcn < -1) {
+		    log_error("Invalid LCN < -1 in mapping pairs array.");
+		    return 0;
+		  }
+		  if(verbose>1)
+		  {
+		    log_verbose("LCN %ld\n",lcn);
+		  }
+		  if(attr_type==my_type)
+		    return lcn;
+		}
+	      }
+	    }
             break;
         }
       }
@@ -389,7 +395,7 @@ static int ntfs_get_attr_aux(const char *attr_record, const int my_type, partiti
   }
 }
 
-static int ntfs_read_MFT(disk_t *disk_car, partition_t *partition, const struct ntfs_boot_sector*ntfs_header,const int my_type,const int verbose, const int dump_ind)
+static int ntfs_read_MFT(disk_t *disk_car, partition_t *partition, const struct ntfs_boot_sector*ntfs_header, const int my_type, const int verbose)
 {
   unsigned char *buffer;
   char *attr;
@@ -415,7 +421,7 @@ static int ntfs_read_MFT(disk_t *disk_car, partition_t *partition, const struct
     return 1;
   }
   buffer=(unsigned char *)MALLOC(mft_size);
-  if(disk_car->read(disk_car,mft_size, buffer, mft_pos)!=0)
+  if((unsigned)disk_car->pread(disk_car, buffer, mft_size, mft_pos) != mft_size)
   {
     log_error("NTFS: Can't read MFT\n");
     free(buffer);
@@ -424,7 +430,7 @@ static int ntfs_read_MFT(disk_t *disk_car, partition_t *partition, const struct
   attr=(char*)buffer;
   while(attr+0x30<=(char*)(buffer+mft_size))
   {
-    int res=ntfs_get_attr(attr,my_type,partition,(char*)buffer+mft_size,verbose,dump_ind,NULL);
+    int res=ntfs_get_attr(attr, my_type, partition, (char*)buffer+mft_size, verbose, NULL);
     if((res>0)|| (NTFS_GETU32(attr + 0x1C)<0x30))
     {
       free(buffer);
