@@ -67,6 +67,7 @@
 #endif
 
 #include "list.h"
+#include "list_sort.h"
 #include "log.h"
 #include "log_part.h"
 #include "ntfs_udl.h"
@@ -912,9 +913,17 @@ static int undelete_file(ntfs_volume *vol, long long inode)
 
 	td_list_for_each(item, &file->data) {
 	  char pathname[256];
+	  char defname[64];
 		struct data *d = td_list_entry(item, struct data, list);
-
-		name = file->pref_name;
+		if(file->pref_name)
+		{
+		  name = file->pref_name;
+		}
+		else
+		{
+		  sprintf(defname, "inode_%llu", (long long unsigned)file->inode);
+		  name = defname;
+		}
 
 		//dir_data->local_dir;
 		create_pathname(opts.dest, file->pref_pname, name, d->name, pathname, sizeof(pathname));
@@ -1073,20 +1082,21 @@ free:
 	return result;
 }
 
-static file_info_t *ufile_to_file_data(const struct ufile *file)
+static file_info_t *ufile_to_file_data(const struct ufile *file, const struct data *d)
 {
-  file_info_t *new_file;
-  if(file->pref_name==NULL)
-    return NULL;
-  new_file=(file_info_t *)MALLOC(sizeof(*new_file));
-  if(file->pref_pname==NULL)
-    new_file->name=strdup(file->pref_name);
-  else
-  {
-    const unsigned int len=strlen(file->pref_name) + 1 + strlen(file->pref_pname) + 1;
-    new_file->name=(char *)MALLOC(len);
-    sprintf(new_file->name, "%s/%s", file->pref_pname, file->pref_name);
-  }
+  file_info_t *new_file=(file_info_t *)MALLOC(sizeof(*new_file));
+  char inode_name[32];
+  const unsigned int len=(file->pref_pname==NULL?0:strlen(file->pref_pname)) +
+    (file->pref_name==NULL?sizeof(inode_name):strlen(file->pref_name) + 1) +
+    (d->name==NULL?0:strlen(d->name) + 1) + 1;
+  sprintf(inode_name, "inode_%llu", (long long unsigned)file->inode);
+  new_file->name=(char *)MALLOC(len);
+  sprintf(new_file->name, "%s%s%s%s%s",
+      (file->pref_pname?file->pref_pname:""),
+      (file->pref_pname?"/":""),
+      (file->pref_name?file->pref_name:inode_name),
+      (d->name?":":""),
+      (d->name?d->name:""));
   new_file->stat.st_dev=0;
   new_file->stat.st_ino=file->inode;
   new_file->stat.st_mode = (file->directory ?LINUX_S_IFDIR| LINUX_S_IRUGO | LINUX_S_IXUGO:LINUX_S_IFREG | LINUX_S_IRUGO);
@@ -1094,9 +1104,10 @@ static file_info_t *ufile_to_file_data(const struct ufile *file)
   new_file->stat.st_uid=0;
   new_file->stat.st_gid=0;
   new_file->stat.st_rdev=0;
-  new_file->stat.st_size=file->max_size;
+
+  new_file->stat.st_size=max(d->size_init, d->size_data);
 #ifdef DJGPP
-  new_file->file_size=file->max_size;
+  new_file->file_size=max(d->size_init, d->size_data);
 #endif
 #ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
   new_file->stat.st_blksize=512;	/* FIXME cluster_size */
@@ -1177,12 +1188,17 @@ static void scan_disk(ntfs_volume *vol, file_info_t *dir_list)
 			  percent = calc_percentage(file, vol);
 			  if (percent >0)
 			  {
-			    file_info_t *new_file;
-			    new_file=ufile_to_file_data(file);
-			    if(new_file!=NULL)
+			    struct td_list_head *item;
+			    td_list_for_each(item, &file->data)
 			    {
-			      td_list_add_sorted(&new_file->list, &dir_list->list, filesort);
-			      results++;
+			      const struct data *d = td_list_entry(item, struct data, list);
+			      file_info_t *new_file;
+			      new_file=ufile_to_file_data(file, d);
+			      if(new_file!=NULL)
+			      {
+				td_list_add_tail(&new_file->list, &dir_list->list);
+				results++;
+			      }
 			    }
 			  }
 			  free_file(file);
@@ -1194,6 +1210,7 @@ done:
 	free(buffer);
 	if (attr)
 		ntfs_attr_close(attr);
+	td_list_sort(&dir_list->list, filesort);
 }
 
 #ifdef HAVE_NCURSES
@@ -1250,11 +1267,11 @@ static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *part
 	  } else {
 	    strncpy(datestr, "                 ",sizeof(datestr));
 	  }
-	  if(COLS <= 1+17+1+7+1)
+	  if(COLS <= 1+17+1+9+1)
 	    wprintw(window, "%s", file_info->name);
 	  else
 	  {
-	    const unsigned int nbr=COLS - (1+17+1+7+1);
+	    const unsigned int nbr=COLS - (1+17+1+9+1);
 	    if(strlen(file_info->name) < nbr)
 	      wprintw(window, "%-*s", nbr, file_info->name);
 	    else
