@@ -40,28 +40,24 @@
 #include "intrf.h"
 #include "chgtype.h"
 #include "savehdr.h"
+#include "analyse.h"
 #include "bfs.h"
 #include "bsd.h"
-#include "cramfs.h"
 #include "exfat.h"
-#include "ext2.h"
 #include "fat.h"
 #include "hfs.h"
 #include "hfsp.h"
-#include "jfs_superblock.h"
-#include "jfs.h"
-#include "luks.h"
 #include "lvm.h"
 #include "md.h"
 #include "netware.h"
 #include "ntfs.h"
-#include "rfs.h"
 #include "sun.h"
 #include "swap.h"
-#include "xfs.h"
 #include "log.h"
 #include "parti386.h"
+#include "partgpt.h"
 
+#define TAB_PART 0x1BE
 static int is_extended(const unsigned int part_type);
 static int test_structure_i386(list_part_t *list_part);
 #define pt_offset_const(b, n) ((const struct partition_dos *)((b) + 0x1be + \
@@ -208,6 +204,7 @@ static const struct systypes i386_sys_types[] = {
   {0xf1,		"Storage Dimensions"},
   {0xf2,		"DOS secondary"},
   {0xf4,		"SpeedStor"},
+  {P_VMFS,		"VMFS"},
   {P_RAID,		"Linux RAID"},
   {0xfe,		"LANstep"},
   {0xff,		"Xenix bad block"},
@@ -264,7 +261,7 @@ static uint64_t get_start_sect(const struct partition_dos *p)
   return read4_little_endian(p->start4);
 }
 
-uint64_t get_nr_sects(const struct partition_dos *p)
+static uint64_t get_nr_sects(const struct partition_dos *p)
 {
   return read4_little_endian(p->size4);
 }
@@ -280,7 +277,7 @@ static void set_start_sect(struct partition_dos *p, unsigned int start_sect)
 }
 
 
-int get_geometry_from_i386mbr(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry)
+static int get_geometry_from_i386mbr(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry)
 {
   unsigned int i;
   if(verbose>1)
@@ -355,7 +352,6 @@ static list_part_t *init_part_order_i386(const disk_t *disk_car, list_part_t *li
 static list_part_t *read_part_i386(disk_t *disk_car, const int verbose, const int saveheader)
 {
   unsigned int i;
-  int res=0;
   CHSgeometry_t geometry;
   list_part_t *new_list_part=NULL;
   unsigned char *buffer=(unsigned char *)MALLOC(disk_car->sector_size);
@@ -399,7 +395,6 @@ static list_part_t *read_part_i386(disk_t *disk_car, const int verbose, const in
 	  if(new_partition->errcode!=BAD_NOERR)
 	  {
 	    screen_buffer_add("%s\n",errmsg_i386_entry2partition(new_partition->errcode));
-	    res|=1;
 	  }
 	  new_list_part=insert_new_partition(new_list_part,new_partition, 0, &insert_error);
 	  if(insert_error>0)
@@ -483,12 +478,12 @@ static list_part_t *get_ext_data_i386(disk_t *disk_car, list_part_t *list_part,
   {
     partition_t *partition_ext=partition_main_ext;
     partition_t *partition_next_ext=NULL;
-    unsigned int i;
     unsigned int order=5;
     do
     {
       unsigned char buffer[DEFAULT_SECTOR_SIZE];
       int nb_hidden=0, nb_mb=0, nb_part=0, nb_ext=0, nb_boot=0;
+      unsigned int i;
       if(disk_car->pread(disk_car, &buffer, sizeof(buffer), partition_ext->part_offset) != sizeof(buffer))
 	return list_part;
       if((buffer[0x1FE]!=(unsigned char)0x55)||(buffer[0x1FF]!=(unsigned char)0xAA))
@@ -611,6 +606,33 @@ static list_part_t *get_ext_data_i386(disk_t *disk_car, list_part_t *list_part,
   return list_part;
 }
 
+int recover_i386_logical(disk_t *disk, const unsigned char *buffer, partition_t *partition)
+{
+  const struct partition_dos *p=pt_offset_const(buffer,0);
+  if(partition->arch!=&arch_i386)
+    return 1;
+  if(is_extended(p->sys_ind))
+    p=pt_offset_const(buffer,1);
+  switch(p->sys_ind)
+  {
+    case P_12FAT:
+    case P_16FAT:
+    case P_16FATBD:
+    case P_16FATBD_LBA:
+    case P_NTFS:
+    case P_32FAT:
+    case P_32FAT_LBA:
+      break;
+    default:
+      return 1;
+  }
+  if(partition->part_offset==0)
+    return 1;
+  i386_entry2partition(disk, partition->part_offset, partition, p, STATUS_DELETED, 0, 0, 0);
+  partition->order=NO_ORDER;
+  return 0;
+}
+
 static int test_MBR_over(disk_t *disk_car,list_part_t *list_part)
 {/* Test if partitions overlap */
   int res=0;
@@ -627,7 +649,7 @@ static int test_MBR_over(disk_t *disk_car,list_part_t *list_part)
   return res;
 }
 
-int write_part_i386(disk_t *disk_car, const list_part_t *list_part, const int ro, const int verbose, const int align)
+static int write_part_i386(disk_t *disk_car, const list_part_t *list_part, const int ro, const int verbose, const int align)
 {
   int res=0;
   res+=write_mbr_i386(disk_car,list_part,ro,verbose);
@@ -777,7 +799,6 @@ static int write_all_log_i386(disk_t *disk_car, const list_part_t *list_part, co
   {
     for(element=pos_ext->next;(element!=NULL) && (element->part->status==STATUS_LOG);element=element->next)
     {
-      int j;
       unsigned char buffer[DEFAULT_SECTOR_SIZE];
       unsigned char buffer_org[DEFAULT_SECTOR_SIZE];
       if(verbose>0)
@@ -839,6 +860,7 @@ static int write_all_log_i386(disk_t *disk_car, const list_part_t *list_part, co
       {
         if(verbose>1)
         {
+	  int j;
           for(j=0;j<4;j++)
           {
             const struct partition_dos *p=pt_offset(buffer,j);
@@ -864,9 +886,9 @@ static int write_all_log_i386(disk_t *disk_car, const list_part_t *list_part, co
 
 static int diff(const unsigned char buffer[DEFAULT_SECTOR_SIZE], const unsigned char buffer_org[DEFAULT_SECTOR_SIZE])
 {
-  int j;
   if(memcmp(buffer,buffer_org,DEFAULT_SECTOR_SIZE))
   {
+    int j;
     log_info("\nSectors are different.\n");
     log_info("buffer_org\n");
     for(j=0;j<4;j++)
@@ -936,16 +958,17 @@ static int write_MBR_code_i386_aux(unsigned char *buffer)
     0xbb, 0x00, 0x7c, 0x50, 0xc6, 0x06, 0x8f, 0x07, 0x01, 0xcd, 0x13, 0x58, 0x5e, 0x73, 0x05, 0x4f,
     0x75, 0xb4, 0xeb, 0x93, 0x81, 0x3e, 0xfe, 0x7d, 0x55, 0xaa, 0x75, 0xf6, 0xea, 0x00, 0x7c, 0x00,
     0x00, 0xbe, 0x83, 0x07, 0xb9, 0x0a, 0x00, 0x50, 0xb4, 0x0e, 0x31, 0xdb, 0xac, 0xcd, 0x10, 0xe2,
-    0xfb, 0x58, 0xc3, 0x54, 0x65, 0x73, 0x74, 0x44, 0x69, 0x73, 0x6b, 0x0d, 0x0a, 0x10, 0x00, 0x01,
-    0x00, 0x00, 0x7c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x31, 0x32, 0x33,
-    0x34, 0x46, 0x00, 0x00, 0x41, 0x4e, 0x44, 0x54, 0x6d, 0x62, 0x72, 0x00, 0x02, 0x02, 0x02, 0x1f,
+    0xfb, 0x58, 0xc3,  'T',  'e',  's',  't',  'D',  'i',  's',  'k', 0x0d, 0x0a, 0x10, 0x00, 0x01,
+    0x00, 0x00, 0x7c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  '1',  '2',  '3',
+     '4',  'F', 0x00, 0x00, 0x41,  'N',  'D',  'T',  'm',  'b',  'r', 0x00, 0x02, 0x02, 0x02, 0x1f,
     0xc7, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xa5, 0x01, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x55, 0xaa,
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x55, 0xaa
   };
-  memcpy(buffer,&mbr_code_testdisk,TAB_PART);
+  /* don't overwrite the disk signature at 0x1b8 */
+  memcpy(buffer, &mbr_code_testdisk, 0x1b8);
   buffer[0x1FE]=(unsigned char)0x55;
   buffer[0x1FF]=(unsigned char)0xAA;
   return 0;
@@ -1433,20 +1456,33 @@ static void init_structure_i386(const disk_t *disk_car,list_part_t *list_part, c
   part_free_list_only(new_list_part);
 }
 
-static int erase_list_part_i386(disk_t *disk_car)
+static int erase_list_part_i386(disk_t *disk)
 {
   unsigned char buffer[DEFAULT_SECTOR_SIZE];
-  if(disk_car->pread(disk_car, buffer, DEFAULT_SECTOR_SIZE, (uint64_t)0) != DEFAULT_SECTOR_SIZE)
+  if(disk->pread(disk, buffer, DEFAULT_SECTOR_SIZE, (uint64_t)0) != DEFAULT_SECTOR_SIZE)
   {
     log_error(msg_PART_RD_ERR);
     memset(buffer,0,sizeof(buffer));
   }
   memset(buffer+TAB_PART,0,0x40);
-  if(disk_car->pwrite(disk_car, buffer, DEFAULT_SECTOR_SIZE, (uint64_t)0) != DEFAULT_SECTOR_SIZE)
+  if(disk->pwrite(disk, buffer, DEFAULT_SECTOR_SIZE, (uint64_t)0) != DEFAULT_SECTOR_SIZE)
   {
     return 1;
   }
-  disk_car->sync(disk_car);
+  {
+    /* Erase EFI GPT signature if present */
+    struct gpt_hdr *gpt=(struct gpt_hdr*)MALLOC(disk->sector_size);
+    if((unsigned)disk->pread(disk, gpt, disk->sector_size, disk->sector_size) == disk->sector_size)
+    {
+      if(memcmp(gpt->hdr_sig, GPT_HDR_SIG, 8)==0)
+      {
+	memset(gpt->hdr_sig, 0, 8);
+	disk->pwrite(disk, gpt, disk->sector_size, disk->sector_size);
+      }
+    }
+    free(gpt);
+  }
+  disk->sync(disk);
   return 0;
 }
 
@@ -1480,36 +1516,16 @@ static int check_part_i386(disk_t *disk_car,const int verbose,partition_t *parti
       { screen_buffer_add("Invalid BSD disklabel\n"); }
       break;
     case P_HFS:
-      ret=check_HFS(disk_car,partition,verbose);
+      ret=check_HFS(disk_car, partition, verbose);
       if(ret!=0)
-      {
-	ret=check_HFSP(disk_car,partition,verbose);
-      }
+	ret=check_HFSP(disk_car, partition, verbose);
+      if(ret!=0)
+	screen_buffer_add("No HFS or HFS+ structure\n");
       break;
     case P_LINUX:
-      ret=check_JFS(disk_car, partition);
+      ret=check_linux(disk_car, partition, verbose);
       if(ret!=0)
-      {
-	ret=check_rfs(disk_car,partition,verbose);
-      }
-      if(ret!=0)
-      {
-	ret=check_EXT2(disk_car,partition,verbose);
-      }
-      if(ret!=0)
-      {
-	ret=check_cramfs(disk_car,partition,verbose);
-      }
-      if(ret!=0)
-      {
-	ret=check_xfs(disk_car,partition,verbose);
-      }
-      if(ret!=0)
-      {
-	ret=check_LUKS(disk_car,partition);
-      }
-      if(ret!=0)
-      { screen_buffer_add("No EXT2, JFS, Reiser, cramfs or XFS marker\n"); }
+	screen_buffer_add("No ext2, JFS, Reiser, cramfs or XFS marker\n");
       break;
     case P_LINSWAP:
       ret=check_Linux_SWAP(disk_car, partition);
@@ -1517,9 +1533,9 @@ static int check_part_i386(disk_t *disk_car,const int verbose,partition_t *parti
     case P_LVM:
       ret=check_LVM(disk_car,partition,verbose);
       if(ret!=0)
-      {
 	ret=check_LVM2(disk_car,partition,verbose);
-      }
+      if(ret!=0)
+	screen_buffer_add("No LVM or LVM2 structure\n");
       break;
     case P_NETBSD:
       ret=check_BSD(disk_car,partition,verbose,BSD_MAXPARTITIONS);
@@ -1540,7 +1556,7 @@ static int check_part_i386(disk_t *disk_car,const int verbose,partition_t *parti
     case P_RAID:
       ret=check_MD(disk_car,partition,verbose);
       if(ret!=0)
-      { screen_buffer_add("Invalid RAID superblock\n"); }
+	screen_buffer_add("Invalid RAID superblock\n");
       break;
     case P_SUN:
       ret=check_sun_i386(disk_car,partition,verbose);
