@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
  
+#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -34,36 +35,25 @@
 #include <ctype.h>      /* tolower */
 #include "types.h"
 #include "common.h"
-#include "testdisk.h"
 #include "fnctdsk.h"
-//#include "analyse.h"
 #include "lang.h"
 #include "intrf.h"
-#include "intrfn.h"
+#include "analyse.h"
 #include "chgtype.h"
 #include "sun.h"
 #include "swap.h"
-#include "bsd.h"
-#include "fat.h"
-#include "ntfs.h"
-#include "ext2.h"
-#include "rfs.h"
 #include "lvm.h"
 #include "md.h"
-#include "jfs_superblock.h"
-#include "jfs.h"
 #include "savehdr.h"
-#include "cramfs.h"
-#include "xfs.h"
 #include "ufs.h"
 #include "log.h"
+#include "partsun.h"
 
 static int check_part_sun(disk_t *disk_car, const int verbose,partition_t *partition,const int saveheader);
-static int get_geometry_from_sunmbr(const unsigned char *buffer, const int verbose, CHS_t *geometry);
+static int get_geometry_from_sunmbr(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry);
 static list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int saveheader);
 static int write_part_sun(disk_t *disk_car, const list_part_t *list_part, const int ro , const int verbose, const int align);
 static list_part_t *init_part_order_sun(const disk_t *disk_car, list_part_t *list_part);
-static list_part_t *add_partition_sun(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd);
 static void set_next_status_sun(const disk_t *disk_car, partition_t *partition);
 static int test_structure_sun(list_part_t *list_part);
 static int set_part_type_sun(partition_t *partition, unsigned int part_type_sun);
@@ -103,16 +93,15 @@ arch_fnct_t arch_sun=
   .get_geometry_from_mbr=get_geometry_from_sunmbr,
   .check_part=check_part_sun,
   .write_MBR_code=NULL,
-  .add_partition=add_partition_sun,
   .set_prev_status=set_next_status_sun,
   .set_next_status=set_next_status_sun,
   .test_structure=test_structure_sun,
+  .get_part_type=get_part_type_sun,
   .set_part_type=set_part_type_sun,
-  .is_part_known=is_part_known_sun,
   .init_structure=init_structure_sun,
   .erase_list_part=NULL,
   .get_partition_typename=get_partition_typename_sun,
-  .get_part_type=get_part_type_sun
+  .is_part_known=is_part_known_sun
 };
 
 static unsigned int get_part_type_sun(const partition_t *partition)
@@ -120,32 +109,33 @@ static unsigned int get_part_type_sun(const partition_t *partition)
   return partition->part_type_sun;
 }
 
-int get_geometry_from_sunmbr(const unsigned char *buffer, const int verbose, CHS_t *geometry)
+static int get_geometry_from_sunmbr(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry)
 {
-  const sun_partition *sunlabel=(const sun_partition*)buffer;
+  const sun_disklabel *sunlabel=(const sun_disklabel*)buffer;
   if(verbose>1)
   {
     log_trace("get_geometry_from_sunmbr\n");
   }
-  geometry->cylinder=0;
-  geometry->head=be16(sunlabel->ntrks)-1;
-  geometry->sector=be16(sunlabel->nsect);
-  if(geometry->sector>0)
+  geometry->cylinders=0;
+  geometry->heads_per_cylinder=be16(sunlabel->ntrks);
+  geometry->sectors_per_head=be16(sunlabel->nsect);
+  if(geometry->sectors_per_head>0)
   {
-    log_info("Geometry from SUN MBR: head=%u sector=%u\n",geometry->head+1,geometry->sector);
+    log_info("Geometry from SUN MBR: head=%u sector=%u\n",
+	geometry->heads_per_cylinder, geometry->sectors_per_head);
   }
   return 0;
 }
 
-list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int saveheader)
+static list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int saveheader)
 {
   unsigned int i;
-  sun_partition *sunlabel;
+  sun_disklabel *sunlabel;
   list_part_t *new_list_part=NULL;
   unsigned char *buffer=(unsigned char *)MALLOC(disk_car->sector_size);
   screen_buffer_reset();
-  sunlabel=(sun_partition*)buffer;
-  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
+  sunlabel=(sun_disklabel*)buffer;
+  if(disk_car->pread(disk_car, buffer, DEFAULT_SECTOR_SIZE, (uint64_t)0) != DEFAULT_SECTOR_SIZE)
   {
     screen_buffer_add( msg_PART_RD_ERR);
     free(buffer);
@@ -182,11 +172,10 @@ list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int savehe
 }
 
 static int write_part_sun(disk_t *disk_car, const list_part_t *list_part, const int ro, const int verbose, const int align)
-{ /* TODO: Implement it */
+{
+  /* TODO: Implement it */
   if(ro==0)
-  {
-    not_implemented("write_part_sun");
-  }
+    return -1;
   return 0;
 }
 
@@ -223,16 +212,16 @@ static list_part_t *init_part_order_sun(const disk_t *disk_car, list_part_t *lis
   return list_part;
 }
 
-static list_part_t *add_partition_sun_cli(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd)
+list_part_t *add_partition_sun_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd)
 {
   CHS_t start,end;
   partition_t *new_partition=partition_new(&arch_sun);
   start.cylinder=0;
   start.head=0;
   start.sector=1;
-  end.cylinder=disk_car->CHS.cylinder;
-  end.head=disk_car->CHS.head;
-  end.sector=disk_car->CHS.sector;
+  end.cylinder=disk_car->geom.cylinders-1;
+  end.head=disk_car->geom.heads_per_cylinder-1;
+  end.sector=disk_car->geom.sectors_per_head;
   while(*current_cmd[0]==',')
     (*current_cmd)++;
   while(1)
@@ -240,17 +229,17 @@ static list_part_t *add_partition_sun_cli(disk_t *disk_car,list_part_t *list_par
     if(strncmp(*current_cmd,"c,",2)==0)
     {
       (*current_cmd)+=2;
-      start.cylinder=ask_number_cli(current_cmd, start.cylinder,0,disk_car->CHS.cylinder,"Enter the starting cylinder ");
+      start.cylinder=ask_number_cli(current_cmd, start.cylinder,0,disk_car->geom.cylinders-1,"Enter the starting cylinder ");
     }
     else if(strncmp(*current_cmd,"C,",2)==0)
     {
       (*current_cmd)+=2;
-      end.cylinder=ask_number_cli(current_cmd, end.cylinder,start.cylinder,disk_car->CHS.cylinder,"Enter the ending cylinder ");
+      end.cylinder=ask_number_cli(current_cmd, end.cylinder,start.cylinder,disk_car->geom.cylinders-1,"Enter the ending cylinder ");
     }
     else if(strncmp(*current_cmd,"T,",2)==0)
     {
       (*current_cmd)+=2;
-      change_part_type(disk_car,new_partition,current_cmd);
+      change_part_type_cli(disk_car,new_partition,current_cmd);
     }
     else if((CHS2offset(disk_car,&end)>new_partition->part_offset) &&
       new_partition->part_type_sun>0)
@@ -275,99 +264,6 @@ static list_part_t *add_partition_sun_cli(disk_t *disk_car,list_part_t *list_par
   }
 }
 
-#ifdef HAVE_NCURSES
-static list_part_t *add_partition_sun_ncurses(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd)
-{
-  CHS_t start,end;
-  partition_t *new_partition=partition_new(&arch_sun);
-  int position=0;
-  start.cylinder=0;
-  start.head=0;
-  start.sector=1;
-  end.cylinder=disk_car->CHS.cylinder;
-  end.head=disk_car->CHS.head;
-  end.sector=disk_car->CHS.sector;
-  {
-    int done = FALSE;
-    while (done==FALSE) {
-      int command;
-      static struct MenuItem menuGeometry[]=
-      {
-	{ 'c', "Cylinders", 	"Change starting cylinder" },
-	{ 'C', "Cylinders", 	"Change ending cylinder" },
-	{ 'T' ,"Type",		"Change partition type"},
-	{ 'd', "Done", "" },
-	{ 0, NULL, NULL }
-      };
-      aff_copy(stdscr);
-      wmove(stdscr,4,0);
-      wprintw(stdscr,"%s",disk_car->description(disk_car));
-      new_partition->part_offset=CHS2offset(disk_car,&start);
-      new_partition->part_size=CHS2offset(disk_car,&end) - new_partition->part_offset + disk_car->sector_size;
-      wmove(stdscr,10, 0);
-      wclrtoeol(stdscr);
-      aff_part(stdscr, AFF_PART_BASE, disk_car, new_partition);
-      wmove(stdscr,INTER_GEOM_Y, INTER_GEOM_X);
-      wclrtoeol(stdscr);
-      wrefresh(stdscr);
-      command=wmenuSimple(stdscr,menuGeometry, position);
-      switch (command) {
-	case 'c':
-	  wmove(stdscr, INTER_GEOM_Y, INTER_GEOM_X);
-	  start.cylinder=ask_number(start.cylinder,0,disk_car->CHS.cylinder,"Enter the starting cylinder ");
-	  position=1;
-	  break;
-	case 'C':
-	  wmove(stdscr, INTER_GEOM_Y, INTER_GEOM_X);
-	  end.cylinder=ask_number(end.cylinder,start.cylinder,disk_car->CHS.cylinder,"Enter the ending cylinder ");
-	  position=2;
-	  break;
-	case 'T':
-	case 't':
-	  change_part_type(disk_car, new_partition, current_cmd);
-	  position=3;
-	  break;
-	case key_ESC:
-	case 'd':
-	case 'D':
-	case 'q':
-	case 'Q':
-	  done = TRUE;
-	  break;
-      }
-    }
-  }
-  if((CHS2offset(disk_car,&end)>new_partition->part_offset) &&
-      new_partition->part_type_sun>0)
-  {
-    int insert_error=0;
-    list_part_t *new_list_part=insert_new_partition(list_part, new_partition, 0, &insert_error);
-    if(insert_error>0)
-    {
-      free(new_partition);
-      return new_list_part;
-    }
-    new_partition->status=STATUS_PRIM;
-    if(test_structure_sun(list_part)!=0)
-      new_partition->status=STATUS_DELETED;
-    return new_list_part;
-  }
-  free(new_partition);
-  return list_part;
-}
-#endif
-
-static list_part_t *add_partition_sun(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd)
-{
-  if(*current_cmd!=NULL)
-    return add_partition_sun_cli(disk_car, list_part, verbose, current_cmd);
-#ifdef HAVE_NCURSES
-  return add_partition_sun_ncurses(disk_car, list_part, verbose, current_cmd);
-#else
-  return list_part;
-#endif
-}
-
 static void set_next_status_sun(const disk_t *disk_car, partition_t *partition)
 {
   if(partition->status==STATUS_DELETED)
@@ -425,9 +321,6 @@ static void init_structure_sun(const disk_t *disk_car,list_part_t *list_part, co
       new_list_part=insert_new_partition(new_list_part, element->part, 0, &insert_error);
     }
   }
-#ifdef DEBUG
-  check_list_part(new_list_part);
-#endif
   for(element=new_list_part;element!=NULL;element=element->next)
     element->part->status=STATUS_PRIM;
   if(disk_car->arch->test_structure(new_list_part))
@@ -436,9 +329,6 @@ static void init_structure_sun(const disk_t *disk_car,list_part_t *list_part, co
       element->part->status=STATUS_DELETED;
   }
   part_free_list_only(new_list_part);
-#ifdef DEBUG
-  check_list_part(list_part);
-#endif
 }
 
 static int check_part_sun(disk_t *disk_car,const int verbose,partition_t *partition, const int saveheader)
@@ -456,29 +346,20 @@ static int check_part_sun(disk_t *disk_car,const int verbose,partition_t *partit
       ret=check_ufs(disk_car,partition,verbose);
       break;
     case PSUN_LINUX:
-      ret=check_JFS(disk_car,partition,verbose);
+      ret=check_linux(disk_car, partition, verbose);
       if(ret!=0)
-      {
-	ret=check_rfs(disk_car,partition,verbose);
-      }
-      if(ret!=0)
-      {
-	ret=check_EXT2(disk_car,partition,verbose);
-      }
-      if(ret!=0)
-      {
-	ret=check_cramfs(disk_car,partition,verbose);
-      }
-      if(ret!=0)
-      {
-	ret=check_xfs(disk_car,partition,verbose);
-      }
-      if(ret!=0)
-      { screen_buffer_add("No EXT2, JFS, Reiser, cramfs or XFS marker\n"); }
+	screen_buffer_add("No EXT2, JFS, Reiser, cramfs or XFS marker\n");
       break;
-
     case PSUN_LINSWAP:
-      ret=check_Linux_SWAP(disk_car,partition,verbose);
+      ret=check_Linux_SWAP(disk_car, partition);
+      break;
+    case PSUN_LVM:
+      ret=check_LVM(disk_car,partition,verbose);
+      if(ret!=0)
+	ret=check_LVM2(disk_car,partition,verbose);
+      break;
+    case PSUN_RAID:
+      ret=check_MD(disk_car,partition,verbose);
       break;
     default:
       if(verbose>0)
