@@ -24,7 +24,7 @@ extern "C" {
 #endif
 #ifdef HAVE_NCURSES
 void photorec_info(WINDOW *window, const file_stat_t *file_stats);
-int photorec_progressbar(WINDOW *window, const unsigned int pass, const photorec_status_t status, const uint64_t offset, disk_t *disk_car, partition_t *partition, const unsigned int file_nbr, const time_t elapsed_time, const file_stat_t *file_stats);
+int photorec_progressbar(WINDOW *window, const unsigned int pass, const struct ph_param *params, const uint64_t offset, const time_t current_time);
 #endif
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
