@@ -19,5 +19,14 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
-int do_curses_photorec(int verbose, const char *recup_dir, const list_disk_t *list_disk, file_enable_t *file_enable, char *cmd_device, char**cmd_run);
-void free_search_space(alloc_data_t *list_search_space);
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+int photorec(struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space, const unsigned int carve_free_space_only);
+void interface_file_select(file_enable_t *files_enable, char**current_cmd);
+void interface_options_photorec(struct ph_options *options, char**current_cmd);
+
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
